/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */


#ifndef InstallH
#define InstallH

#include <textui.h>

#include <vesa.h>
#include <mouse.h>
#include <dosdrv.h>
#include <files.h>

#include <dosio.h>

#include <FatInst.h>
#include <FsCreate.h>
#include <Upgrade.h>


#define BOOTITEM_FILESIZE 4096
#define LASTCONF_FILESIZE 512


class CInstaller {
public:
	CInstaller(CTextUI &TextUIToUse, CPartList &PartListToUse);
	~CInstaller();
	int Install(CVesa::TGraphicsMode GraphicsMode, CMouse::TMouseType MouseType, const CDosDriveList::CDosDrive &DosDrive);
	int Install(CVesa::TGraphicsMode GraphicsMode, CMouse::TMouseType MouseType, int PartIndex);

	int Uninstall(const CDosDriveList::CDosDrive &DosDrive);
	int Uninstall(int PartIndex);

	int Upgrade(const CDosDriveList::CDosDrive &DosDrive);
	int Upgrade(int PartIndex);
	
	class CMbr {
	public:
		unsigned short IPL[223];
		unsigned char PartList[15 * 4];
		unsigned long XoslSignature;
		unsigned short MagicNumber; // also used as checksum for MBR backup
	};

private:
	CTextUI &TextUI;
	CPartList &PartList;
	CXoslFiles XoslFiles;
	CDosFile DosFile;

	CFatInstall FatInstall;
	CFsCreator FsCreator;

	CUpgradeData UpgradeData;

	int BackupSpace;

	int CreateXoslData(CVesa::TGraphicsMode GraphicsMode, CMouse::TMouseType MouseType);
	int CreateBootItem();
	int CreateLastConf();

	void BackupMbr();
	void CheckBackupSpaceAvail();


	int RestoreMbr();
	void RemoveXoslFiles(char DosDriveChar);

};

#endif
