/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */


#include <MbrFix.h>
#include <Disk.h>
#include <mem.h>

void puts(const char *str);



void operator delete(void *)
{
}

void *operator new(unsigned int)
{
	return (void *)0;
}

CMbrFix::CMbrFix()
{

}

CMbrFix::~CMbrFix()
{

}

void CMbrFix::FixMbr()
{

	if (LoadXoslIpl() == -1) {
		puts("Unable to fix master boot record\n\rRun INSTALL and use \"Upgrade to XOSL 1.1.0\" instead\n\r");
		return;
	}
	
	if (UpdateMbr() == -1) {
		puts("Unable to fix master boot record\n\rRun INSTALL and use \"Upgrade to XOSL 1.1.0\" instead\n\r");
		return;
	}

	puts("Master boot record fixed.\n\r");
}


int CMbrFix::LoadXoslIpl()
{
	CDisk Disk;
	unsigned short Checksum;
	int Index;

	if (Disk.Map(0x80,0) == -1)
		return -1;

	if (Disk.Read(2,&Mbr,1) == -1)
		return -1;

	for (Checksum = Index = 0; Index < 223; ++Index)
		Checksum += Mbr.IPL[Index];

	if (Mbr.MagicNumber != Checksum || Mbr.XoslSignature != 0x4c534f58)
		return -1;
	
	return 0;
}


int CMbrFix::UpdateMbr()
{
	CDisk Disk;
	CInstaller::CMbr DestMbr;

	if (Disk.Map(0x80,0) == -1)
		return -1;
	if (Disk.Read(0,&DestMbr,1) == -1)
		return -1;

	memcpy(DestMbr.IPL,Mbr.IPL,446);
	Disk.Lock();
	if (Disk.Write(0,&DestMbr,1) == -1) {
		Disk.Unlock();
		return -1;
	}

	Disk.Unlock();
	return 0;
}


void main()
{
	CMbrFix MbrFix;

	puts("XOSL 1.1.0 master boot record fix\n\r");
	puts("Copyright (C) 1999, Geurt Vos\n\n\r");

	MbrFix.FixMbr();
}