/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */



#include <main.h>



void main()
{
	CApplication Application;
}


CApplication::CApplication():
	TextScreen(0x1f00),
	TextUI(TextScreen),
	Data(TextUI,PartList),
	Installer(TextUI,PartList),
	InstallMenus(TextUI,Data,PartList,(CData::GetDosVersion() & 0xff) > 3)
{
	DoExit = 0;
	DoReboot = 0;
	TextUI.OutputStr("Extended Operating System Loader\n");
	TextUI.OutputStr("comes with ABSOLUTELY NO WARRANTY\n\n");

	InstallMenus.InitMainMenu((CTextList::TListItemExecute)MainMenuExecute,this);
	ApplicationLoop();
}

CApplication::~CApplication()
{
	if (DoReboot)
		Reboot();
}


void CApplication::ApplicationLoop()
{
	int Key;

	while (!DoExit) {
		while (!Keyboard.KbHit());
		Key = Keyboard.GetCh();
		if (Key == KEY_ESCAPE)
			DoExit = 1;
		TextUI.HandleKey(Key);
	}

}


int CApplication::StartInstallFat()
{
	CVesa::TGraphicsMode GraphicsMode;
	CMouse::TMouseType MouseType;

	CDosDriveList DosDriveList(PartList);
	CDosDriveList::CDosDrive DosDrive;
	int DosDriveIndex;

	GraphicsMode = Data.GetGraphicsMode(TextUI.GetOptionIndex(0));
	MouseType = Data.GetMouseType(TextUI.GetOptionIndex(1));
	DosDriveIndex = TextUI.GetOptionIndex(2);

	if (DosDriveList.LocateDrive(DosDriveIndex,DosDrive) == -1) {
		TextUI.OutputStr("Unable to locate drive %c:\n",'C' + DosDriveIndex);
		return -1;
	}

	if (Installer.Install(GraphicsMode,MouseType,DosDrive) == -1) {
		TextUI.OutputStr("Install error\n");
		return -1;
	}

	return 0;
}

int CApplication::StartInstallSep()
{
	CVesa::TGraphicsMode GraphicsMode;
	CMouse::TMouseType MouseType;
	int PartIndex;
	
	GraphicsMode = Data.GetGraphicsMode(TextUI.GetOptionIndex(0));
	MouseType = Data.GetMouseType(TextUI.GetOptionIndex(1));

	PartIndex = InstallMenus.ResolvePartIndex(TextUI.GetOptionIndex(3));

	if (Installer.Install(GraphicsMode,MouseType,PartIndex) == -1) {
		TextUI.OutputStr("Install error\n");
		return -1;
	}
	return 0;
}


int CApplication::StartUpgradeFat()
{
	CDosDriveList DosDriveList(PartList);
	CDosDriveList::CDosDrive DosDrive;
	int DosDriveIndex;

	DosDriveIndex = TextUI.GetOptionIndex(0);

	if (DosDriveList.LocateDrive(DosDriveIndex,DosDrive) == -1) {
		TextUI.OutputStr("Unable to locate drive %c:\n",'C' + DosDriveIndex);
		return -1;
	}

	if (Installer.Upgrade(DosDrive) == -1) {
		TextUI.OutputStr("Install error\n");
		return -1;
	}

	return 0;

}

int CApplication::StartUpgradeSep()
{
	int PartIndex;
	
	PartIndex = InstallMenus.ResolvePartIndex(TextUI.GetOptionIndex(0));

	if (Installer.Upgrade(PartIndex) == -1) {
		TextUI.OutputStr("Install error\n");
		return -1;
	}
	return 0;
}


int CApplication::StartUninstallFat()
{
	CDosDriveList DosDriveList(PartList);
	CDosDriveList::CDosDrive DosDrive;
	int DosDriveIndex;

	DosDriveIndex = TextUI.GetOptionIndex(0);

	if (DosDriveList.LocateDrive(DosDriveIndex,DosDrive) == -1) {
		TextUI.OutputStr("Unable to locate drive %c:\n",'C' + DosDriveIndex);
		return -1;
	}
	return Installer.Uninstall(DosDrive);
}

int CApplication::StartUninstallSep()
{
	int PartIndex;
	
	PartIndex = InstallMenus.ResolvePartIndex(TextUI.GetOptionIndex(0));
	return Installer.Uninstall(PartIndex);
}

void CApplication::MainMenuExecute(CApplication *Application, TEnumMainMenu Item)
{
	switch (Item) {
		case enumInstall:
			if (Application->Data.CollectData(1,1,1) == -1)
				Application->InstallMenus.InitErrorMenu((CTextList::TListItemExecute)Application->ErrorMenuExecute,Application);
			else
				Application->InstallMenus.InitInstallMenu((CTextList::TListItemExecute)Application->InstallMenuExecute,Application);
			break;
		case enumUpgrade:
			Application->Data.CollectData(0,0,1);
			Application->InstallMenus.InitUpgradeMenu((CTextList::TListItemExecute)Application->UpgradeMenuExecute,Application);
			break;
		case enumUninstall:
			Application->Data.CollectData(0,0,1);
			Application->InstallMenus.InitUninstallMenu((CTextList::TListItemExecute)Application->UninstallMenuExecute,Application);
			break;
		case enumExit:
			Application->DoExit = 1;
			break;
		default:
			break;
	}
}


void CApplication::InstallMenuExecute(CApplication *Application, TEnumInstallMenu Item)
{
	switch (Item) {
		case enumInstFat:
			Application->InstallMenus.InitInstFatMenu((CTextList::TListItemExecute)Application->InstFatMenuExecute,Application);
			break;
		case enumInstSep:
			Application->InstallMenus.InitInstSepMenu((CTextList::TListItemExecute)Application->InstSepMenuExecute,Application);
			Application->TextUI.OutputStr("WARNING:\nAll data on the partition you installXOSL on will be destroyed!\n\n");
			break;
		case enumInstAbort:
			Application->InstallMenus.InitMainMenu((CTextList::TListItemExecute)Application->MainMenuExecute,Application);
			break;
		default:
			break;
	}
}



void CApplication::UpgradeMenuExecute(CApplication *Application, TEnumInstallMenu Item)
{
	switch (Item) {
		case enumInstFat:
			Application->InstallMenus.InitUpgradeFat((CTextList::TListItemExecute)Application->UpgradeFatExecute,Application);
			break;
		case enumInstSep:
			Application->InstallMenus.InitUpgradeSep((CTextList::TListItemExecute)Application->UpgradeSepExecute,Application);
			break;
		case enumInstAbort:
			Application->InstallMenus.InitMainMenu((CTextList::TListItemExecute)Application->MainMenuExecute,Application);
			break;
		default:
			break;
	}
}


void CApplication::UninstallMenuExecute(CApplication *Application, TEnumInstallMenu Item)
{
	switch (Item) {
		case enumInstFat:
			Application->InstallMenus.InitUninstallFat((CTextList::TListItemExecute)Application->UninstallFatExecute,Application);
			break;
		case enumInstSep:
			Application->InstallMenus.InitUninstallSep((CTextList::TListItemExecute)Application->UninstallSepExecute,Application);
			break;
		case enumInstAbort:
			Application->InstallMenus.InitMainMenu((CTextList::TListItemExecute)Application->MainMenuExecute,Application);
			break;
		default:
			break;
	}
}




void CApplication::InstFatMenuExecute(CApplication *Application, TEnumInstFatMenu Item)
{
	switch (Item) {
		case enumInstFatStart:
			if (Application->StartInstallFat() == -1)
				Application->InstallMenus.InitErrorMenu((CTextList::TListItemExecute)Application->ErrorMenuExecute,Application);
			else
				Application->InstallMenus.InitDoneMenu((CTextList::TListItemExecute)Application->DoneMenuExecute,Application);
			break;
		case enumInstFatAbort:
			Application->InstallMenus.InitInstallMenu((CTextList::TListItemExecute)Application->InstallMenuExecute,Application);
			break;
		default:
			break;
	}
}


void CApplication::InstSepMenuExecute(CApplication *Application, TEnumInstSepMenu Item)
{
	switch (Item) {
		case enumInstSepStart:
			if (Application->StartInstallSep() == -1)
				Application->InstallMenus.InitErrorMenu((CTextList::TListItemExecute)Application->ErrorMenuExecute,Application);
			else
				Application->InstallMenus.InitDoneMenu((CTextList::TListItemExecute)Application->DoneMenuExecute,Application);
			break;
		case enumInstSepAbort:
			Application->InstallMenus.InitInstallMenu((CTextList::TListItemExecute)Application->InstallMenuExecute,Application);
			break;
		default:
			break;
	}
}


void CApplication::ErrorMenuExecute(CApplication *Application, TEnumErrorMenu Item)
{
	switch (Item) {
		case enumErrorMain:
			Application->InstallMenus.InitMainMenu((CTextList::TListItemExecute)Application->MainMenuExecute,Application);
			break;
		case enumErrorQuit:
			Application->DoExit = 1;
			break;
		default:
			break;
	}
}

void CApplication::DoneMenuExecute(CApplication *Application, TEnumDoneMenu Item)
{
	switch (Item) {
		case enumDoneReboot:
			Application->DoReboot = 1;
		case enumDoneExit:
			Application->DoExit = 1;
			break;
		case enumDoneMain:
			Application->InstallMenus.InitMainMenu((CTextList::TListItemExecute)Application->MainMenuExecute,Application);
			break;
		default:
			break;
	}
}

void CApplication::UninstallFatExecute(CApplication *Application, TEnumFatUninstallMenu Item)
{
	switch (Item) {
		case enumFatUnStart:
			if (Application->StartUninstallFat() == -1)
				Application->InstallMenus.InitErrorMenu((CTextList::TListItemExecute)Application->ErrorMenuExecute,Application);
			else
				Application->InstallMenus.InitUnDoneMenu((CTextList::TListItemExecute)Application->DoneMenuExecute,Application);			
			break;
		case enumFatUnAbort:
			Application->InstallMenus.InitUninstallMenu((CTextList::TListItemExecute)Application->UninstallMenuExecute,Application);
			break;
		default:
			break;
	}
}

void CApplication::UninstallSepExecute(CApplication *Application, TEnumSepUninstallMenu Item)
{
	switch (Item) {
		case enumSepUnStart:
			if (Application->StartUninstallSep() == -1)
				Application->InstallMenus.InitErrorMenu((CTextList::TListItemExecute)Application->ErrorMenuExecute,Application);
			else
				Application->InstallMenus.InitUnDoneMenu((CTextList::TListItemExecute)Application->DoneMenuExecute,Application);			
			break;
		case enumSepUnAbort:
			Application->InstallMenus.InitUninstallMenu((CTextList::TListItemExecute)Application->UninstallMenuExecute,Application);
			break;
		default:
			break;
	}
}


void CApplication::UpgradeFatExecute(CApplication *Application, TEnumFatUninstallMenu Item)
{
	switch (Item) {
		case enumFatUnStart:
			if (Application->StartUpgradeFat() == -1)
				Application->InstallMenus.InitErrorMenu((CTextList::TListItemExecute)Application->ErrorMenuExecute,Application);
			else
				Application->InstallMenus.InitDoneMenu((CTextList::TListItemExecute)Application->DoneMenuExecute,Application);
			break;
		case enumFatUnAbort:
			Application->InstallMenus.InitUpgradeMenu((CTextList::TListItemExecute)Application->UpgradeMenuExecute,Application);
			break;
		default:
			break;
	}
}

void CApplication::UpgradeSepExecute(CApplication *Application, TEnumSepUninstallMenu Item)
{
	switch (Item) {
		case enumSepUnStart:
			if (Application->StartUpgradeSep() == -1)
				Application->InstallMenus.InitErrorMenu((CTextList::TListItemExecute)Application->ErrorMenuExecute,Application);
			else
				Application->InstallMenus.InitDoneMenu((CTextList::TListItemExecute)Application->DoneMenuExecute,Application);
			break;
		case enumSepUnAbort:
			Application->InstallMenus.InitUpgradeMenu((CTextList::TListItemExecute)Application->UpgradeMenuExecute,Application);
			break;
		default:
			break;
	}
}
