/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */


#include "menus.h"
#include <string.h>
#include <mem.h>



CInstallMenus::CInstallMenus(CTextUI &TextUIToUse, CData &DataToUse, CPartList &PartListToUse, int FatAvailToUse):
	TextUI(TextUIToUse),
	Data(DataToUse),
	PartList(PartListToUse)
{
	FatAvail = FatAvailToUse;
	DosDriveList = NULL;
	PartNameList = NULL;
}

CInstallMenus::~CInstallMenus()
{
	if (DosDriveList) {
		delete DosDriveList[0];
		delete DosDriveList;
	}
}

void CInstallMenus::InitMainMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Install XOSL 1.1.0","Install XOSL 1.1.0",1);
	TextUI.AddMenuItem(2,"Upgrade to XOSL 1.1.0","Upgrade XOSL 1.0.0 or 1.1.0.dv1 to XOSL 1.1.0",1);
	TextUI.AddMenuItem(4,"Uninstall XOSL","Uninstall Extended Operating System Loader (up to version 1.1.0)",1);
	TextUI.AddMenuItem(6,"Exit install","Quit without installing Extended Operating System Loader 1.1.0",1);
	TextUI.SetItemIndex(0);
	TextUI.RefreshMenu();
}

void CInstallMenus::InitInstallMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Install on FAT16/FAT32 partition","Install XOSL 1.1.0 on an existing FAT16 or FAT32 partition",FatAvail);
	TextUI.AddMenuItem(2,"Install on dedicated partition","Install XOSL 1.1.0 on an existing dedicated partition",1);
	TextUI.AddMenuItem(6,"Return to main menu","Return to main menu",1);
	TextUI.SetItemIndex(FatAvail ? 0 : 2);
	TextUI.RefreshMenu();
}

void CInstallMenus::InitUpgradeMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"XOSL on FAT16/FAT32 partition","Upgrade XOSL  on a FAT16 or FAT32 partition",FatAvail);
	TextUI.AddMenuItem(2,"XOSL on dedicated partition","Upgrade XOSL on a dedicated partition",1);
	TextUI.AddMenuItem(6,"Return to main menu","Return to main menu",1);
	TextUI.SetItemIndex(FatAvail ? 0 : 2);
	TextUI.RefreshMenu();
}

void CInstallMenus::InitUninstallMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"XOSL on FAT16/FAT32 partition","Uninstall XOSL off a FAT16 or FAT32 partition",FatAvail);
	TextUI.AddMenuItem(2,"XOSL on dedicated partition","Uninstall XOSL off a dedicated partition",1);
	TextUI.AddMenuItem(6,"Return to main menu","Return to main menu",1);
	TextUI.SetItemIndex(FatAvail ? 0 : 2);
	TextUI.RefreshMenu();
}

void CInstallMenus::InitInstFatMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	const char **ModeNameList;
	int ModeCount;
	const char **MouseTypeNames;
	int MouseTypeCount;

	Data.GetGraphicsModeNames(ModeNameList,ModeCount);
	Data.GetMouseTypeNames(MouseTypeNames,MouseTypeCount);
	if (!DosDriveList)
		CreateDosDriveList();

	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Video mode","Initial video mode",1,ModeCount,ModeNameList,1);
	TextUI.AddMenuItem(1,"Mouse type","Initial mouse type",1,MouseTypeCount,MouseTypeNames,1);

	TextUI.AddMenuItem(2,"Install on drive","Drive to install XOSL on",1,DosDriveCount,(const char **)DosDriveList,1);
	TextUI.AddMenuItem(4,"Start installation","Install Extended Operating System Loader 1.1.0",1);
	TextUI.AddMenuItem(6,"Return to install menu","Return to install menu",1);
	TextUI.SetItemIndex(0);
	TextUI.RefreshMenu();
}

void CInstallMenus::InitInstSepMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	const char **ModeNameList;
	int ModeCount;
	const char **MouseTypeNames;
	int MouseTypeCount;

	Data.GetGraphicsModeNames(ModeNameList,ModeCount);
	Data.GetMouseTypeNames(MouseTypeNames,MouseTypeCount);
	if (!PartNameList)
		CreatePartList();

	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Video mode","Initial video mode",1,ModeCount,ModeNameList,1);
	TextUI.AddMenuItem(1,"Mouse type","Initial mouse type",1,MouseTypeCount,MouseTypeNames,1);
	TextUI.AddMenuItem(3,"Drv Type System              Size ","Partition to install XOSL 1.1.0 on",1,PartNameCount,(const char **)PartNameList,0);
	TextUI.AddMenuItem(8,"Start installation","Install Extended Operating System Loader 1.1.0",1);
	TextUI.AddMenuItem(10,"Return to install menu","Return to install menu",1);
	TextUI.SetItemIndex(0);
	TextUI.RefreshMenu();
}


void CInstallMenus::InitErrorMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"An error has occured!","",0);
	TextUI.AddMenuItem(2,"Exit install","Quit without installing Extended Operating System Loader 1.1.0",1);
	TextUI.AddMenuItem(4,"Return to main menu","Return to main menu",1);
	TextUI.SetItemIndex(2);
	TextUI.RefreshMenu();
}

void CInstallMenus::InitDoneMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Extended Operating System Loader","",0);
	TextUI.AddMenuItem(1,"has been successfully installed","",0);

	TextUI.AddMenuItem(3,"Reboot system","Quit install and reboot to run XOSL for the first time",1);
	TextUI.AddMenuItem(5,"Exit install","Quit install without rebooting",1);
	TextUI.AddMenuItem(7,"Return to main menu","Return to main menu",1);
	TextUI.SetItemIndex(3);
	TextUI.RefreshMenu();
}

void CInstallMenus::InitUnDoneMenu(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);
	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Extended Operating System Loader","",0);
	TextUI.AddMenuItem(1,"has been successfully uninstalled","",0);

	TextUI.AddMenuItem(3,"Reboot system","Quit install and reboot",1);
	TextUI.AddMenuItem(5,"Exit install","Quit install",1);
	TextUI.AddMenuItem(7,"Return to main menu","Return to main menu",1);
	TextUI.SetItemIndex(3);
	TextUI.RefreshMenu();
}

void CInstallMenus::InitUninstallFat(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	if (!DosDriveList)
		CreateDosDriveList();

	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);

	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Uninstall off drive","Drive XOSL is currently installed on",1,DosDriveCount,(const char **)DosDriveList,1);
	TextUI.AddMenuItem(4,"Start uninstall","Uninstall Extended Operating System Loader",1);
	TextUI.AddMenuItem(6,"Return to uninstall menu","Return to uninstall menu",1);
	TextUI.SetItemIndex(0);
	TextUI.RefreshMenu();

}

void CInstallMenus::InitUninstallSep(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	if (!PartNameList)
		CreatePartList();

	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);

	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Drv Type System              Size ","Partition XOSL is currently installed on",1,PartNameCount,(const char **)PartNameList,0);
	TextUI.AddMenuItem(6,"Start uninstall","Uninstall Extended Operating System Loader",1);
	TextUI.AddMenuItem(8,"Return to uninstall menu","Return to uninstall menu",1);
	TextUI.SetItemIndex(0);
	TextUI.RefreshMenu();

}

void CInstallMenus::InitUpgradeFat(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	if (!DosDriveList)
		CreateDosDriveList();

	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);

	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Upgrade on drive","Drive XOSL is currently installed on",1,DosDriveCount,(const char **)DosDriveList,1);
	TextUI.AddMenuItem(4,"Start upgrade","Upgrade existing installation of XOSL to version 1.1.0",1);
	TextUI.AddMenuItem(6,"Return to upgrade menu","Return to upgrade menu",1);
	TextUI.SetItemIndex(0);
	TextUI.RefreshMenu();

}

void CInstallMenus::InitUpgradeSep(CTextList::TListItemExecute MenuHandler, void *HandlerClass)
{
	if (!PartNameList)
		CreatePartList();

	TextUI.ConnectEventHandler(MenuHandler,HandlerClass);

	TextUI.ClearMenu();
	TextUI.AddMenuItem(0,"Drv Type System              Size ","Partition XOSL is currently installed on",1,PartNameCount,(const char **)PartNameList,0);
	TextUI.AddMenuItem(6,"Start upgrade","Upgrade existing installation of XOSL to version 1.1.0",1);
	TextUI.AddMenuItem(8,"Return to upgrade menu","Return to upgrade menu",1);
	TextUI.SetItemIndex(0);
	TextUI.RefreshMenu();

}


void CInstallMenus::CreateDosDriveList()
{
	int Index;
	int *Buffer;

	DosDriveCount = Data.GetLastDrive() - ('C' - 'A');
	DosDriveList = new char *[DosDriveCount];
	Buffer = new int[DosDriveCount];
	for (Index = 0; Index < DosDriveCount; ++Index) {
		Buffer[Index] = Index + 'C';
		DosDriveList[Index] = (char *)&Buffer[Index];
	}
}

void CInstallMenus::CreatePartList()
// TODO: Implement the sprintf() function
{
	char *PartStr;
	const TPartition *Partition;
	int Index, Count;

	Count = PartList.GetCount();
	PartNameList = new char *[Count];
	PartResolveList = new int[Count];

	for (PartNameCount = Index = 0; Index < Count; ++Index) {
		Partition = PartList.GetPartition(Index);
		switch (Partition->Type) {
			case PART_PRIMARY:
			case PART_LOGICAL:
				switch (Partition->FSType) {
					case 0x05: // Extended
					case 0x0f: // Extended LBA
					case 0x85: // Linux Extended
						PartResolveList[Index] = -1;
						break;
					default:
						PartResolveList[Index] = PartNameCount;

						PartStr = new char [48];
						memset(PartStr,' ',32);
						memcpy(PartStr,"HD",2);
						PartStr[2] = '0' + (Partition->Drive & ~0x80);
						memcpy(&PartStr[4],Partition->Type == PART_PRIMARY ? "pri" : "log",3);
						memcpy(&PartStr[9],Partition->FSName,strlen(Partition->FSName));
						itoa(Partition->SectorCount >> 11,&PartStr[29],10);
						PartNameList[PartNameCount++] = PartStr;
						break;
				}
				break;
			default:
				break;
		}
	}

}

int CInstallMenus::ResolvePartIndex(int ListIndex)
{
	int Count, Index;

	Count = PartList.GetCount();
	for (Index = 0; Index < Count; ++Index)
		if (PartResolveList[Index] == ListIndex)
			return Index;
	return -1;
}
