/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */


#ifndef UpgradeH
#define UpgradeH

#include <Ptab.h>
#include <fs.h>
#include <textui.h>
#include <files.h>
#include <dosio.h>

#define XOSL_DATA_SIZE 512
#define BOOT_ITEM_SIZE 4096
#define LAST_CONF_SIZE_OLD 2048 // XOSL 1.0.0 & 1.1.0.dv1
#define LAST_CONF_SIZE_NEW 512  // XOSL 1.1.0



class CUpgradeData {
public:
	CUpgradeData(CTextUI &TextUIToUse, CXoslFiles &XoslFilesToUse, CDosFile &DosFileToUse, CPartList &PartListToUse);
	~CUpgradeData();

	CreateDataFiles(char DriveNumber);
	CreateDataFiles(const TPartition *Partition);

	int ConvertData();

private:
	CTextUI &TextUI;
	CXoslFiles &XoslFiles;
	CDosFile &DosFile;
	CPartList &PartList;

	char XoslDataBuffer[XOSL_DATA_SIZE * 2]; 
	char BootItemBuffer[BOOT_ITEM_SIZE];
	char LastConfBuffer[LAST_CONF_SIZE_OLD];

	int InstalledVersion;

	int LoadDataFiles(char DriveNumber);

	int LoadDataFiles(CFileSystem &FileSystem);

	int SaveDataFiles();


	int Upgrade110Data();
	int Upgrade100Data();
	int Upgrade110dv1Data();

	int LoadFile(const char *FileName, int FileSize, void *Buffer);
	int SaveFile(const char *FileName, int FileSize, void *Buffer);

	void GetFat16FileName(char *Fat16Name, const char *DosName);

};

#endif
