/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <backgnd.h>
#include <graph.h>
#include <rgb.h>
#include <cstring.h>

static const char *Title = "Extended Operating System Loader 1.1.0";
static const char *Support = "Linear frame buffer supported";
static const char *NotSupport = "Linear frame buffer not supported";

CBackground::CBackground()
{
}

CBackground::~CBackground()
{
}

void CBackground::SplashScreen(int Width, int Height)
{
	int Left, Top;

	Graph->QBar(0,0,Width,Height,0xfe);
	Left = Width - SplashLogoWidth >> 1;
	Top = Height - SplashLogoHeight >> 1;
	Graph->QPutImage(Left,Top,SplashLogoWidth,SplashLogoHeight,SplashLogo);
}


void CBackground::NewGraphicsMode(int Width, int Height, int UseWallpaper)
{
	int X, Y;
	int TitleLeft, TitleTop;

	if (UseWallpaper)
		for (Y = 0; Y < Height; Y += WallpaperHeight)
			for (X = 0; X < Width; X += WallpaperWidth)
				Graph->QPutImage(X,Y,WallpaperWidth,WallpaperHeight,Wallpaper);
	else
		Graph->QBar(0,0,Width,Height,16);

	TitleLeft = Width - Graph->GetTextWidth(Title,STYLE_REGULAR) - 2;
	TitleTop = Height - Graph->GetTextHeight() - 2;
	Graph->QTextOut(TitleLeft - 1,TitleTop,Title,STYLE_REGULAR,0);
	Graph->QTextOut(TitleLeft + 1,TitleTop,Title,STYLE_REGULAR,0);
	Graph->QTextOut(TitleLeft,TitleTop - 1,Title,STYLE_REGULAR,0);
	Graph->QTextOut(TitleLeft,TitleTop + 1,Title,STYLE_REGULAR,0);
	Graph->QTextOut(TitleLeft - 1,TitleTop - 1,Title,STYLE_REGULAR,0);
	Graph->QTextOut(TitleLeft - 1,TitleTop + 1,Title,STYLE_REGULAR,0);
	Graph->QTextOut(TitleLeft + 1,TitleTop - 1,Title,STYLE_REGULAR,0);
	Graph->QTextOut(TitleLeft + 1,TitleTop + 1,Title,STYLE_REGULAR,0);
	Graph->QTextOut(TitleLeft,TitleTop,Title,STYLE_REGULAR,15);
	Graph->QStoreBuffer(0,0,Width,Height);
}

void CBackground::DrawTestScreen()
{
	DrawTestBlock(1600,1200,2);
	DrawTestBlock(1280,1024,1);
	DrawTestBlock(1024,768,4);
	DrawTestBlock(800,600,2);
	DrawTestBlock(640,480,1);
	Graph->QHLine(0,0,20,15);
	Graph->QVLine(0,0,20,15);
}

void CBackground::DrawTestBlock(int Width, int Height, int Color)
{
	int Step, Index;
	int TextLeft, TextTop;
	TGraphMode GraphMode;
	const char *ModeName;

	Graph->QBar(0,0,Width,Height,Color);
	Step = Height / 10;
	for (Index = 0; Index + 10 < Height; Index += Step) {
		Graph->QHLine(Width - 20,Index,20,15);
		Graph->QVLine(Width - 1,Index - 10,20,15);
	}
	Step = Width / 10;
	for (Index = 0; Index + 10 < Width; Index += Step) {
		Graph->QVLine(Index,Height - 20,20,15);
		Graph->QHLine(Index - 10, Height - 1,20,15);
	}
	GraphMode = Graph->GetModeFromMetrics(Width,Height);
	ModeName = Graph->GetModeName(GraphMode);
	TextTop = Height - Graph->GetTextHeight() - 30;
	TextLeft = Width - Graph->GetTextWidth(ModeName,STYLE_BOLD) >> 1;
	Graph->QTextOut(TextLeft,TextTop,ModeName,STYLE_BOLD,0);

}

void CBackground::FixDamage(int Left, int Top, int Width, int Height)
{
	Graph->QRestoreBuffer(Left,Top,Width,Height);
}

void CBackground::SetSplashLogo(int Width, int Height, const unsigned char *SplashLogo)
{
	this->SplashLogo = SplashLogo;
	SplashLogoWidth = Width;
	SplashLogoHeight = Height;
}

void CBackground::SetWallpaper(int Width, int Height, const unsigned char *Wallpaper)
{
	this->Wallpaper = Wallpaper;
	WallpaperWidth = Width;
	WallpaperHeight = Height;
}
