/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <button.h>
#include <graph.h>
#include <key.h>

CButton::CButton(const char *Caption, int Left, int Top,
	int Width, int Height, int Visible, void *HandlerClass):
	CWindow(Left,Top,Width,Height,Visible,FALSE,TRUE,HandlerClass)
{
	this->Caption = Caption;
	ButtonDown = FALSE;
	DrawDown = FALSE;
	FontStyle = STYLE_REGULAR;
}

CButton::~CButton()
{
}

void CButton::SetFontStyle(int FontStyle)
{
	this->FontStyle = FontStyle;
}

int CButton::MouseDown(int MouseX, int MouseY)
{
	int Status;
	int HadFocus;

	HadFocus = GotFocus;
	DrawDown = TRUE;
	Status = CWindow::MouseDown(MouseX,MouseY);
	if (Status != -1 && Enabled) {
		ButtonDown = TRUE;
		if (HadFocus)
			Refresh();
	}
	else
		DrawDown = FALSE;
	return Status;
}

void CButton::MouseMove(int X, int Y)
{
	CWindow::MouseMove(X,Y);
	if (ButtonDown)
		if (MouseX < Left || MouseX > Right || MouseY < Top || MouseY > Bottom) {
			if (DrawDown) {
				DrawDown = FALSE;
				Refresh();
			}
		}
		else
			if (!DrawDown) {
				DrawDown = TRUE;
				Refresh();
			}
}

void CButton::MouseUp()
{
	ButtonDown = FALSE;
	if (DrawDown) {
		DrawDown = FALSE;
		Refresh();
		CWindow::MouseUp();
	}
}

void CButton::Draw(long, long, long, long)
{
	int XCap, YCap;

	XCap = (Width - Graph->GetTextWidth(Caption,FontStyle)) >> 1;
	YCap = (Height - Graph->GetTextHeight()) >> 1;

	if (GotFocus) {
		Graph->QRectangle(0,0,Width,Height,17);
		if (DrawDown) {
			Graph->QBar(2,2,Width - 4,Height - 4,19);
			Graph->QRectangle(1,1,Width - 2,Height - 2,18);
			Graph->QTextOut(XCap + 1,YCap + 1,Caption,FontStyle,17);
			Graph->QRectangle(5,5,Width - 8,Height - 8,18);
		}
		else {
			Graph->QBar(3,3,Width - 6,Height - 6,19);
			if (Enabled)
				Graph->QTextOut(XCap,YCap,Caption,FontStyle,17);
			else {
				Graph->QTextOut(XCap + 1,YCap + 1,Caption,FontStyle,21);
				Graph->QTextOut(XCap,YCap,Caption,FontStyle,18);
			}
			Graph->QRectangle(0,0,Width,Height,17);
			Graph->QHLine(1,1,Width - 3,21);
			Graph->QVLine(1,2,Height - 4,21);
			Graph->QHLine(2,2,Width - 5,20);
			Graph->QVLine(2,3,Height - 6,20);
			Graph->QHLine(1,Height - 2,Width - 2,17);
			Graph->QVLine(Width - 2,1,Height - 3,17);
			Graph->QHLine(2,Height - 3,Width - 4,18);
			Graph->QVLine(Width - 3,2,Height - 5,18);
//			Graph->QTextOut(XCap,YCap,Caption,STYLE_REGULAR,17);
			Graph->QRectangle(4,4,Width - 8,Height - 8,18);
		}
	}
	else {
		Graph->QBar(2,2,Width - 4,Height - 4,19);
		Graph->QHLine(0,0,Width - 1,21);
		Graph->QVLine(0,1,Height - 2,21);
		Graph->QHLine(1,1,Width - 3,20);
		Graph->QVLine(1,2,Height - 4,20);
		Graph->QHLine(0,Height - 1,Width,17);
		Graph->QVLine(Width - 1,0,Height - 1,17);
		Graph->QHLine(1,Height - 2,Width - 2,18);
		Graph->QVLine(Width - 2,1,Height - 3,18);
		if (Enabled)
			Graph->QTextOut(XCap,YCap,Caption,FontStyle,17);
		else {
			Graph->QTextOut(XCap + 1,YCap + 1,Caption,FontStyle,21);
			Graph->QTextOut(XCap,YCap,Caption,FontStyle,18);
		}
	}
}

void CButton::KeyPress(int Key)
{
	if (ButtonDown)
		return;
	if (WndOnKeyPress && HandlerClass)
		WndOnKeyPress(HandlerClass,Key);
	switch (Key) {
		case KEY_ENTER:
		case KEY_SPACE:
			MouseDown(Left,Top);
			MouseUp();
			break;
	}
}


