/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <form.h>
#include <graph.h>
#include <key.h>

int CForm::FrameMove = 1;


CForm::CForm(const char *Caption, int Style, int OnTop, int Left, int Top,
	int Width, int Height, int Visible):
	CWindow(Left,Top,Width,Height,Visible,OnTop,TRUE,NULL)
{

	this->Caption = Caption;
	this->Style = Style;
	ButtonDown = FALSE;

	Controls = new CWindowList();
	Controls->SetHandler(this);
	Controls->SetVisible(FALSE);
   IsDrawn = FALSE;

	if (Style == FORM_NORMAL) {
		Controls->SetPosition(Left + 2,Top + 22);
		Controls->SetMetrics(Width - 4,Height - 24);
	}
	else {
		Controls->SetPosition(Left,Top);
		Controls->SetMetrics(Width,Height);
	}

	IgnoreAfter = FALSE;
}

CForm::~CForm()
{
	delete Controls;
}

void CForm::Show()
{
	if (Visible)
		return;
	Controls->SetVisible(TRUE);
	Parent->SetFocusWnd(this,TRUE);
	CWindow::Show();
}

void CForm::Hide()
{
	if (!Visible)
		return;
	Controls->SetVisible(FALSE);
	CWindow::Hide();
}

void CForm::SetMetrics(int Width, int Height)
{
	CWindow::SetMetrics(Width,Height);
	if (Style == FORM_NORMAL)
		Controls->SetMetrics(Width - 4,Height - 24);
	else
		Controls->SetMetrics(Width,Height);
}

void CForm::SetPosition(int Left, int Top)
{
	if (Style == FORM_NORMAL)
		Controls->SetPosition(Left + 2,Top + 22);
	else
		Controls->SetPosition(Left,Top);
	CWindow::SetPosition(Left,Top);
}

void CForm::FixDamage(int Left, int Top, int Width, int Height)
{
	int iAbsLeft, iAbsTop;
	long AbsLeft, AbsTop;
	long VPLeft, VPTop;
	long ClipLeft,ClipTop,ClipWidth,ClipHeight;
	long lLeft, lTop, lWidth, lHeight;

	if (!IsDrawn) {
		IsDrawn = TRUE;
		Controls->SetVisible(TRUE);
	}
	if (Visible && AdjustArea(Left,Top,Width,Height) != -1 && Parent) {
		PreFocus = FALSE;
		Graph->QGetClippingRegion(ClipLeft,ClipTop,ClipWidth,ClipHeight);
		Graph->QGetViewportOrigin(VPLeft,VPTop);

		GetAbsPosition(iAbsLeft,iAbsTop);
		AbsLeft = iAbsLeft;
		AbsTop = iAbsTop;

		Graph->QSetViewportOrigin(AbsLeft,AbsTop);

		Left -= this->Left;
		Top -= this->Top;

		lLeft = AbsLeft + Left;
		lTop = AbsTop + Top;
		lWidth = Width;
		lHeight = Height;
		Graph->QSetClippingRegion(lLeft,lTop,lWidth,lHeight);

		Draw(Left,Top,Width,Height);

		Graph->QSetViewportOrigin(VPLeft,VPTop);
		Graph->QSetClippingRegion(ClipLeft,ClipTop,ClipWidth,ClipHeight);

		Graph->FlushQueue();

		IgnoreAfter = TRUE;
		if (Style == FORM_NORMAL)
			Controls->FixDamage(Left - 2,Top - 22,Width,Height);
		else
			Controls->FixDamage(Left,Top,Width,Height);
		IgnoreAfter = FALSE;
	}
}

void CForm::BeforeFix(int Left, int Top, int Width, int Height)
{
	if (Style == FORM_NORMAL)
		Graph->QBar(Left,Top,Width,Height,19);
}

void CForm::AfterFix(int Left, int Top, int Width, int Height)
{
	if ((!OnTop || Parent->GetFocusWnd() != this) && !IgnoreAfter)
		Parent->FixDamage(Left,Top,Width,Height);
}

int CForm::MouseDown(int MouseX, int MouseY)
{
	int Status;

	Status = CWindow::MouseDown(MouseX,MouseY);
	if (Status != -1 && Enabled)
		if (Style == FORM_NORMAL &&
			 MouseX >= Left && MouseX <= Right &&
			 MouseY >= Top && MouseY <= Top + 21) {
			/**/
			ButtonDown = TRUE;

			if (FrameMove) {
				DistX = Left - MouseX;
				DistY = Top - MouseY;

				LeftFrame = new char[3 * Height + 6];
				RightFrame = new char[3 * Height+ 6];
				TopFrame = new char[3 * Width+ 6];
				BottomFrame = new char[3 * Width+ 6];

				oMoveLeft = MoveLeft = MouseX + DistX;
				oMoveTop = MoveTop = MouseY + DistY;
				oMoveRight = MoveRight = MoveLeft + Width - 1;
				oMoveBottom = MoveBottom = MoveTop + Height - 1;

            Graph->HideCursor();
				ShowFrame();
			}
			/**/
		}
		else
			Controls->MouseDown(MouseX,MouseY);
	return Status;
}

void CForm::MouseMove(int X, int Y)
{
	if (ButtonDown) {
		/**/

		if (!FrameMove) {
			SetPosition(Left + X - MouseX,Top + Y - MouseY);
		}
		else {
			HideFrame();
			MoveLeft = X + DistX;
			MoveTop = Y  + DistY;
			MoveRight = MoveLeft + Width - 1;
			MoveBottom = MoveTop + Height - 1;
			ShowFrame();
		}
		CWindow::MouseMove(X,Y);
		/**/
	}
	else {
		CWindow::MouseMove(X,Y);
		Controls->MouseMove(X,Y);
	}
}

void CForm::MouseUp()
{
	if (ButtonDown) {
		/**/
		ButtonDown = FALSE;
		if (FrameMove) {
			HideFrame();
			FlushFrame();
			SetPosition(MouseX + DistX,MouseY + DistY);
			Graph->ShowCursor();
			delete LeftFrame;
			delete RightFrame;
			delete TopFrame;
			delete BottomFrame;
		}
		/**/
	}
	else
		Controls->MouseUp();
}


//void CForm::Draw(long Left, long Top, long Width, long Height)
void CForm::Draw(long, long, long, long)
{
	int lTB;
	int TextTop;

	if (Style == FORM_TRANSP)
		return;
	Graph->QHLine(0,0,Width - 1,20);
	Graph->QVLine(0,1,Height - 2,20);

	Graph->QHLine(1,1,Width - 2,21);
	Graph->QVLine(1,2,Height - 3,21);

	Graph->QHLine(0,Height - 1,Width,17);
	Graph->QVLine(Width - 1,0,Height - 1,17);

	Graph->QHLine(1,Height - 2,Width - 2,18);
	Graph->QVLine(Width - 2,1,Height - 3,18);

	Graph->QVLine(2,2,20,19);
	Graph->QVLine(Width - 3,2,20,19);

	Graph->QVLine(3,3,18,18);
	Graph->QHLine(3,2,Width - 7,18);

	Graph->QVLine(Width - 4,3,19,21);
	Graph->QHLine(3,21,Width - 6,21);

	TextTop = 22 - Graph->GetTextHeight() >> 1;
	if (GotFocus) {
		for (lTB = 3; lTB < 21; ++lTB)
			Graph->QHLine(4,lTB,Width - 8,lTB + 19);
		Graph->QTextOut(7,TextTop,Caption,STYLE_BOLD,21);
	}
	else {
		Graph->QBar(4,3,Width - 8,18,18);
		Graph->QTextOut(7,TextTop,Caption,STYLE_BOLD,19);
	}

}

void CForm::AddControl(CWindow *Control)
{
	Controls->Add(Control);
}

void CForm::RemoveControl(CWindow *Control)
{
	Controls->Remove(Control);
}

void CForm::FocusControl(CWindow *Control)
{
	Controls->SetFocusWnd(Control,FALSE);
}

void CForm::KeyPress(int Key)
{
	CWindow *Wnd;

	switch (Key) {
		case KEY_TAB:
			Controls->FocusNext();
			break;
		case KEY_STAB:
			Controls->FocusPrev();
			break;
		default:
			Wnd = Controls->GetFocusWnd();
			if (Wnd && Wnd->Visible)
				Wnd->KeyPress(Key);
			break;
	}
}

void CForm::ShowFrame()
{
	Graph->QGetImage(MoveLeft - 1,MoveTop - 1,3,Height + 2,LeftFrame);
	Graph->QGetImage(MoveLeft - 1,MoveTop - 1,Width + 2,3,TopFrame);
	Graph->QGetImage(MoveRight - 1,MoveTop - 1,3,Height + 2,RightFrame);
	Graph->QGetImage(MoveLeft - 1,MoveBottom - 1,Width + 2,3,BottomFrame);

	Graph->QRectangle(MoveLeft,MoveTop,Width,Height,15);
	Graph->QRectangle(MoveLeft - 1,MoveTop - 1,Width + 2,Height + 2,0);
	Graph->QRectangle(MoveLeft  + 1,MoveTop + 1,Width - 2,Height - 2,0);
	FlushFrame();
}

void CForm::HideFrame()
{
	Graph->QPutImage(MoveLeft - 1,MoveTop - 1,3,Height + 2,LeftFrame);
	Graph->QPutImage(MoveLeft - 1,MoveTop - 1,Width + 2,3,TopFrame);
	Graph->QPutImage(MoveRight - 1,MoveTop - 1,3,Height + 2,RightFrame);
	Graph->QPutImage(MoveLeft - 1,MoveBottom - 1,Width + 2,3,BottomFrame);

	oMoveLeft = MoveLeft;
	oMoveTop = MoveTop;
	oMoveRight = MoveRight;
	oMoveBottom = MoveBottom;
}

void CForm::FlushFrame()
{
//	Graph->FlushQueue();

	Graph->QFlushArea(MoveLeft - 1,MoveTop - 1,3,Height + 2);
	Graph->QFlushArea(MoveLeft - 1,MoveTop - 1,Width + 2,3);
	Graph->QFlushArea(MoveRight - 1,MoveTop - 1,3,Height + 2);
	Graph->QFlushArea(MoveLeft - 1,MoveBottom - 1,Width + 2,3);

	Graph->QFlushArea(oMoveLeft - 1,oMoveTop - 1,3,Height + 2);
	Graph->QFlushArea(oMoveLeft - 1,oMoveTop - 1,Width + 2,3);
	Graph->QFlushArea(oMoveRight - 1,oMoveTop - 1,3,Height + 2);
	Graph->QFlushArea(oMoveLeft - 1,oMoveBottom - 1,Width + 2,3);

//	Graph->WaitRetrace();
	Graph->FlushQueue();
}
