/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <trackbar.h>
#include <graph.h>
#include <key.h>

#define TRACKBAR_HEIGHT 15

CTrackBar::CTrackBar(int Min, int Max, int Value,
		int Left, int Top, int Width, int Visible, void *HandlerClass):
		CWindow(Left,Top,Width,TRACKBAR_HEIGHT,Visible,FALSE,TRUE,HandlerClass)
{
	this->Min = Min;
	this->Max = Max;
	this->Value = Value;
	SetSliderPos();
	ButtonDown = FALSE;
	TrackBarChange = NULL;
}

CTrackBar::~CTrackBar()
{
}

void CTrackBar::SetMin(int Min)
{
	this->Min = Min;
	SetSliderPos();
	Refresh();
}

void CTrackBar::SetMax(int Max)
{
	this->Max = Max;
	SetSliderPos();
	Refresh();
}

void CTrackBar::SetValue(int Value)
{
	if (Value > Max)
		Value = Max;
	if (Value < Min)
		Value = Min;
	if (Value != this->Value) {
		this->Value = Value;
		SetSliderPos();
		Refresh();
		if (TrackBarChange && HandlerClass)
			TrackBarChange(HandlerClass,Value);
	}
}

int CTrackBar::GetValue()
{
	return Value;
}

void CTrackBar::Draw(long, long, long, long)
{
	Graph->QHLine(4,4,Width - 9,18);
	Graph->QVLine(4,5,4,18);
	Graph->QHLine(5,5,Width - 11,17);
	Graph->QPutPixel(5,6,17);
	Graph->QPutPixel(5,7,17);
	Graph->QBar(6,6,Width - 12,2,21);
	Graph->QHLine(6,8,Width - 10,20);
	Graph->QVLine(Width - 6,5,3,20);
	Graph->QHLine(4,9,Width - 8,21);
	Graph->QVLine(Width - 5,4,5,21);
	if (GotFocus)
		Graph->QRectangle(0,0,Width,Height,18);
	DrawSlider();
}

void CTrackBar::DrawSlider()
{
	Graph->QHLine(SliderPos,3,4,21);
	Graph->QVLine(SliderPos,4,7,21);
	Graph->QHLine(SliderPos,11,5,17);
	Graph->QVLine(SliderPos + 1,4,6,20);
	Graph->QHLine(SliderPos + 1,10,3,18);
	Graph->QVLine(SliderPos + 2,5,5,19);
	Graph->QVLine(SliderPos + 3,4,6,18);
	Graph->QVLine(SliderPos + 4,3,8,17);
}

void CTrackBar::SetSliderPos()
{
	SliderPos = 4 + ((long)(Value - Min) * (Width - 13)) / (Max - Min);
}

void CTrackBar::SetMetrics(int Width, int Height)
{
	CWindow::SetMetrics(Width,Height);
	SetValue(Value);
}

int CTrackBar::MouseDown(int MouseX, int MouseY)
{
	int Status;

	Status = CWindow::MouseDown(MouseX,MouseY);
	if (Status != -1) {
		ButtonDown = TRUE;
		MouseMove(MouseX,MouseY);
	}
	return Status;
}

void CTrackBar::MouseMove(int X, int)
{
	if (ButtonDown)
		SetValue(((long)(X - Left - 4) * (Max - Min)) / (Width - 13) + Min);
}

void CTrackBar::MouseUp()
{
	ButtonDown = FALSE;
}

void CTrackBar::OnChange(TTrackBarChange TrackBarChange)
{
	this->TrackBarChange = TrackBarChange;
}

void CTrackBar::KeyPress(int Key)
{
	switch (Key) {
		case KEY_NONE:
			break;
		case KEY_LEFT:
			SetValue(Value - 1);
			break;
		case KEY_RIGHT:
			SetValue(Value + 1);
			break;
		case KEY_HOME:
			SetValue(Min);
			break;
		case KEY_END:
			SetValue(Max);
			break;
		case KEY_PAGEUP:
		case KEY_CTRLLEFT:
			SetValue(Value - 8);
			break;
		case KEY_PAGEDOWN:
		case KEY_CTRLRIGHT:
			SetValue(Value + 8);
			break;
		default:
			break;
	}
}
