/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#ifndef __graph__
#define __graph__

#include <newdefs.h>
#include <palette.h>
#include <mouse.h>
#include <..\graph\int10.h>

#define STYLE_REGULAR 0
#define STYLE_BOLD    1

typedef struct {
	int Index;
	unsigned char Items;
	unsigned char Width;
} TFontEntry;

typedef enum { 
	modeText = 0, mode640x480, mode800x600, 
	mode1024x768, mode1280x1024, mode1600x1200
} TGraphMode;


typedef struct {
	const char far *Name; // name of videocard
	int VersionMaj;       // major VBE version number
	int VersionMin;       // minor VBE version number
	int Memory;           // amount of installed videomemory (kb)
	int Width;		       // screen width
	int Height;           // screen height
} TGraphInfo;

class CGraph {
	public:
		CGraph(CMouse *MouseToUse);
		~CGraph();
		void DetectVideoModes();
	private:
		void AddVideoMode(TGraphMode GraphMode, int VesaMode, TModeInfo &ModeInfo);
	public:
		int SetMode(TGraphMode Mode);
		TGraphMode GetMode();

		TGraphMode GetModeFromIndex(int ModeIndex);
      TGraphMode GetModeFromMetrics(int Width, int Height);
		int GetModeIndex(TGraphMode GraphMode);
		int GetModeCount();



		void VBE2(int Enabled);
		int LinearSupport();
		const char *GetModeName();
		const char *GetModeName(int Mode/*TGraphMode Mode*/);
		void GetModeMetrics(int &Width, int &Height);
		void GetGraphInfo(TGraphInfo &GraphInfo);
		int ModeSupported(TGraphMode Mode);

		void FlushQueue();

		void WaitRetrace();

		void QFlushArea(long Left, long Top, long Width, long Height);
		void QUltraFlush();
		void QStoreBuffer(long Left, long Top, long Width, long Height);
		void QRestoreBuffer(long Left, long Top, long Width, long Height);

		void QSetViewportOrigin(long &Left, long &Top);
		void QGetViewportOrigin(long &Left, long &Top);
      void QSetClippingRegion(long &Left, long &Top, long &Width, long &Height);
		void QGetClippingRegion(long &Left, long &Top, long &Width, long &Height);

		void QPutPixel(long Left, long Top, long Color);
		void QHLine(long Left, long Top, long Width, long Color);
		void QVLine(long Left, long Top, long Height, long Color);
		void QRectangle(long Left, long Top, long Width, long Height, long Color);
		void QBar(long Left, long Top, long Width, long Height, long Color);
		void QPutImage(long Left, long Top, long Width, long Height, const void far *Image);
		void QGetImage(long Left, long Top, long Width, long Height, const void far *Image);
		void QLine(long X1, long Y1, long X2, long Y2, long Color);

		void SetIsStack(int Status);


		// Cursor
		void SetCursorXY(int X, int Y);
      void GetCursorXY(int &X, int &Y);
		void ShowCursor();
		void HideCursor();

		// Text
		void SetFont(void *FontFile);
		int GetTextWidth(const char *Str, int Style);
		int GetTextHeight();
		void QTextOut(int Left, int Top, const char *Str, int Style, int Color);


		CPalette *Palette;
	private:
		// Text
		TFontEntry *FontHeader;
		//
		void CreateBankswitch(int Granularity, unsigned long SwitchAddr);
		void QueueAction(void *Func,long far *ParamList);
		long far *CreateParamList(long P1 = 0, long P2 = 0, long P3 = 0, long P4 = 0, long P5 = 0);
		TGraphInfo GraphInfo;
		int LFBSupport;
		unsigned long AddrLinearFB;
		TGraphMode Mode;
		const char *ModeName;
		int IsStack;

		CMouse *Mouse;
};

extern CGraph *Graph;

#endif