/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#ifndef __wndlist__
#define __wndlist__

#include <newdefs.h>

class CWindow;
class CScreen;
class CForm;


typedef struct SWndNode {
	CWindow *Wnd;
	struct SWndNode *Next;
	struct SWndNode *Prev;
} TWndNode, *PWndNode;

class CWindowList {
	public:
		CWindowList();
		~CWindowList();
		void SetVisible(int Visible);
		int GetVisible();

		void Add(CWindow *Wnd);
		void Remove(CWindow *Wnd);
		CWindow *Items(int Index);
		void Delete(int Index);
		int Count();

		void SetPosition(int Left, int Top);
		void GetPosition(int &Left, int &Top);
		void SetMetrics(int Width, int Height);
		void GetMetrics(int &Width, int &Height);

		void SetFocusWnd(CWindow *Wnd, int MoveFocused);
		CWindow *GetFocusWnd();
		void FocusNext();
		void FocusPrev();
      CWindow *GetLastFocus();

		void FixDamage(long Left, long Top, long Width, long Height);
		void SetIgnoreFix(int Status);

		void SetHandler(CScreen *hScreen);
		void SetHandler(CForm *hForm);

		void MouseDown(int Left, int Top);
		void MouseUp();
		void MouseMove(int Left, int Top);

	private:
		void Insert(int Index, CWindow *Wnd);
		PWndNode Locate(CWindow *Wnd);
		void SwitchFocus(CWindow *From, CWindow *To);
		int AdjustArea(long &iLeft, long &iTop, long &iWidth, long &iHeight);

		void GotoFirst();
		void GotoLast();
		CWindow *GetThis();
		CWindow *GetNext();
		CWindow *GetPrev();
		PWndNode LastNode;
		PWndNode ThisNode;

		PWndNode WindowList;
		int WndCount;

		long Left, Top;
		long Width, Height;
		int Visible;

		PWndNode FocusEntry;
		CWindow *LastFocus;

		CScreen *hScreen;
		CForm *hForm;

};

#endif