/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <about.h>
#include <loader.h>

extern unsigned char *SplashLogo;

static const char *StrXoslTitle = "Extended Operating System Loader";
static const char *StrCopyright = "Copyright  1999/2000, Geurt Vos";
static const char *StrHttpXosl = "http://www.xosl.org";
static const char *StrLogoInfo = "XOSL logo created by:";
static const char *StrLogoCreator = "Mark Monciardini";
static const char *StrHttpLogo = "http://www.designsbymark.com";
static const char *StrCreateInfo = "Created by: Geurt Vos";
static const char *StrCreateInfo2 = "Created by:";
static const char *StrDesignInfo = "Additional design: Dan Duskin";
static const char *StrDesignInfo2 = "Additional design:";
static const char *StrThanksInfo1 = "Thanks to: Mikhail Ranish, Pavel Rezny";
static const char *StrThanksInfo12 = "Thanks to:";
static const char *StrThanksInfo2 = "and everyone else who have helped me";
static const char *StrThanksInfo3 = "with the development of XOSL.";
static const char *StrWarranty = "XOSL comes with ABSOLUTELY NO WARRANTY";


CAbout::CAbout(CLoader *LoaderToUse, CScreen *ScreenToUse)
{
	Loader = LoaderToUse;
	Screen = ScreenToUse;
	CreateWidgets();
	InitWidgets();
	InstallWidgets();
}

CAbout::~CAbout()
{
}

void CAbout::Show()
{
	if (!Form->Visible) {
		Form->Show();
		Form->FocusControl(CloseBtn);
	}
	else
		Screen->SetFocusWnd(Form);
}

void CAbout::Hide()
{
	if (Form->Visible)
		Form->Hide();
}

int CAbout::IsVisible()
{
	return Form->Visible;
}

int CAbout::GotFocus()
{
	return Screen->GetFocusWnd() == Form;
}

void CAbout::CreateWidgets()
{
	int lLeft;

	Form = new CForm("About XOSL",FORM_NORMAL,TRUE,200,10,469,308,FALSE);
	Image = new CImage(SplashLogo,218,146,1,7,7,220,148,TRUE);

	lLeft = 232 + ((225 - Graph->GetTextWidth(StrXoslTitle,STYLE_BOLD)) >> 1);
	XoslTitle = new CLabel(StrXoslTitle,STYLE_BOLD,TRUE,17,lLeft,25,TRUE);
	lLeft = 232 + ((225 - Graph->GetTextWidth(StrCopyright,STYLE_REGULAR)) >> 1);
	Copyright = new CLabel(StrCopyright,STYLE_REGULAR,TRUE,17,lLeft,41,TRUE);
	lLeft = 232 + ((225 - Graph->GetTextWidth(StrHttpXosl,STYLE_REGULAR)) >> 1);
	HttpXosl = new CLabel(StrHttpXosl,STYLE_REGULAR,TRUE,22,lLeft,57,TRUE);
	lLeft = 232 + ((225 - Graph->GetTextWidth(StrLogoInfo,STYLE_REGULAR)) >> 1);
	LogoInfo = new CLabel(StrLogoInfo,STYLE_REGULAR,TRUE,17,lLeft,89,TRUE);
	lLeft = 232 + ((225 - Graph->GetTextWidth(StrLogoCreator,STYLE_REGULAR)) >> 1);
	LogoCreator = new CLabel(StrLogoCreator,STYLE_REGULAR,TRUE,17,lLeft,105,TRUE);
	lLeft = 232 + ((225 - Graph->GetTextWidth(StrHttpLogo,STYLE_REGULAR)) >> 1);
	HttpLogo = new CLabel(StrHttpLogo,STYLE_REGULAR,TRUE,22,lLeft,121,TRUE);
	CreateInfo = new CLabel(StrCreateInfo,STYLE_REGULAR,TRUE,17,8,160,TRUE);
	CreateInfo2 = new CLabel(StrCreateInfo2,STYLE_REGULAR,TRUE,17,9,160,TRUE);
	DesignInfo = new CLabel(StrDesignInfo,STYLE_REGULAR,TRUE,17,8,184,TRUE);
	DesignInfo2 = new CLabel(StrDesignInfo2,STYLE_REGULAR,TRUE,17,9,184,TRUE);
	ThanksInfo1 = new CLabel(StrThanksInfo1,STYLE_REGULAR,TRUE,17,8,208,TRUE);
	ThanksInfo12 = new CLabel(StrThanksInfo12,STYLE_REGULAR,TRUE,17,9,208,TRUE);
	ThanksInfo2 = new CLabel(StrThanksInfo2,STYLE_REGULAR,TRUE,17,8,224,TRUE);
	ThanksInfo3 = new CLabel(StrThanksInfo3,STYLE_REGULAR,TRUE,17,8,240,TRUE);
	Warranty = new CLabel(StrWarranty,STYLE_REGULAR,TRUE,17,8,268,TRUE);
	
	CloseBtn = new CButton("Close",307,204,75,25,TRUE,this);
}

void CAbout::InitWidgets()
{
	CloseBtn->OnClick((TWndOnClick)CloseBtnClick);
}

void CAbout::InstallWidgets()
{
	Form->AddControl(Image);
	Form->AddControl(XoslTitle);
	Form->AddControl(Copyright);
	Form->AddControl(HttpXosl);
	Form->AddControl(LogoInfo);
	Form->AddControl(LogoCreator);
	Form->AddControl(HttpLogo);
	Form->AddControl(CreateInfo);
	Form->AddControl(CreateInfo2);
	Form->AddControl(DesignInfo);
	Form->AddControl(DesignInfo2);
	Form->AddControl(ThanksInfo1);
	Form->AddControl(ThanksInfo12);
	Form->AddControl(ThanksInfo2);
	Form->AddControl(ThanksInfo3);
	Form->AddControl(Warranty);
	Form->AddControl(CloseBtn);
	Screen->AddWindow(Form);
}

void CAbout::CenterWindow()
{
	int ScrnWidth, ScrnHeight;
	int Width, Height;

	Graph->GetModeMetrics(ScrnWidth,ScrnHeight);
	Form->GetMetrics(Width,Height);
	Form->SetPosition((ScrnWidth - Width) >> 1,(ScrnHeight - Height) >> 1);
}

void CAbout::RealignText()
{
	int Left, Top;

	XoslTitle->SetVisible(FALSE);
	Copyright->SetVisible(FALSE);
	HttpXosl->SetVisible(FALSE);
	LogoInfo->SetVisible(FALSE);
	LogoCreator->SetVisible(FALSE);
	HttpLogo->SetVisible(FALSE);

	XoslTitle->GetPosition(Left,Top);
	Left = 232 + ((225 - Graph->GetTextWidth(StrXoslTitle,STYLE_BOLD)) >> 1);
	XoslTitle->SetPosition(Left,Top);
	Copyright->GetPosition(Left,Top);
	Left = 232 + ((225 - Graph->GetTextWidth(StrCopyright,STYLE_REGULAR)) >> 1);
	Copyright->SetPosition(Left,Top);
	HttpXosl->GetPosition(Left,Top);
	Left = 232 + ((225 - Graph->GetTextWidth(StrHttpXosl,STYLE_REGULAR)) >> 1);
	HttpXosl->SetPosition(Left,Top);
	LogoInfo->GetPosition(Left,Top);
	Left = 232 + ((225 - Graph->GetTextWidth(StrLogoInfo,STYLE_REGULAR)) >> 1);
	LogoInfo->SetPosition(Left,Top);
	LogoCreator->GetPosition(Left,Top);
	Left = 232 + ((225 - Graph->GetTextWidth(StrLogoCreator,STYLE_REGULAR)) >> 1);
	LogoCreator->SetPosition(Left,Top);
	HttpLogo->GetPosition(Left,Top);
	Left = 232 + ((225 - Graph->GetTextWidth(StrHttpLogo,STYLE_REGULAR)) >> 1);
	HttpLogo->SetPosition(Left,Top);

	XoslTitle->SetVisible(TRUE);
	Copyright->SetVisible(TRUE);
	HttpXosl->SetVisible(TRUE);
	LogoInfo->SetVisible(TRUE);
	LogoCreator->SetVisible(TRUE);
	HttpLogo->SetVisible(TRUE);
}


void CAbout::CloseBtnClick(CAbout *About)
{
	About->Hide();
	About->Loader->Focus();
}
