/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <graph.h>
#include <gui.h>
#include <screen.h>
#include <text.h>
#include <dialogs.h>
#include <key.h>
#include <quit.h>

#define ClientHeight 105


CDialogs::CDialogs(CScreen *ScreenToUse, CForm *Parent, void *HandlerClass)
{
	this->Parent = Parent;
	this->HandlerClass = HandlerClass;
	Screen = ScreenToUse;

	Form = new CForm("",FORM_NORMAL,TRUE,0,0,204,ClientHeight + 24,FALSE);
	BodyText = new CLabel("",STYLE_REGULAR,TRUE,17,16,24,TRUE);
	BodyEdit = new CEdit("",32,TRUE,FALSE,16,26,168,FALSE,this);
	BodyEdit->OnKeyPress((TWndOnKeyPress)PwdKeyPress);
	CloseBtn = new CButton("Ok",62,64,75,25,TRUE,this);
	CloseBtn->OnClick((TWndOnClick)CloseBtnClick);
	Form->AddControl(BodyText);
	Form->AddControl(BodyEdit);
	Form->AddControl(CloseBtn);
	Screen->AddWindow(Form);
}

CDialogs::~CDialogs()
{
}

void CDialogs::ShowMessageDialog(const char *Title, const char *Text)
{
	DialogType = dtMessage;
	ShowMessageBox(Title,Text,"Ok");
}

void CDialogs::ShowPasswordDialog(const char *Title, TPwdProc PwdProcToUse)
{
	int ScrnWidth, ScrnHeight;
	int BoxLeft, BoxTop;

	DialogType = dtPassword;
	PwdProc = PwdProcToUse;

	Graph->GetModeMetrics(ScrnWidth,ScrnHeight);
	BoxLeft = (ScrnWidth - 204) >> 1;
	BoxTop = (ScrnHeight - ClientHeight - 24) >> 1;
	if (Parent)
		Parent->Disable();
	Form->SetVisible(FALSE);
	Form->SetCaption(Title);
	Form->SetPosition(BoxLeft,BoxTop);
	Form->SetMetrics(204,ClientHeight + 32);
	BodyText->Hide();
	BodyEdit->Show();
	BodyEdit->SetText("");
	CloseBtn->SetCaption("Ok");
	CloseBtn->SetPosition(62,72);
	Form->FocusControl(BodyEdit);
	Form->Show();
}

void CDialogs::ShowAlertDialog(const char *Title, const char *Text,
	const char *ButtonCaption, TAlertProc AlertProcToUse)
{
	DialogType = dtAlert;
	AlertProc = AlertProcToUse;
	ShowMessageBox(Title,Text,ButtonCaption);
}

void CDialogs::CloseBtnClick(CDialogs *Dialogs)
{
	char Password[33];

	Dialogs->Form->Hide();
	if (Dialogs->Parent) {
		Dialogs->Parent->Enable();
		Dialogs->Screen->SetFocusWnd(Dialogs->Parent);
	}
	switch (Dialogs->DialogType) {
		case dtMessage:
			break;
		case dtPassword:
			if (Dialogs->PwdProc) {
				Dialogs->BodyEdit->GetText(Password);
				Dialogs->PwdProc(Dialogs->HandlerClass,Password);
			}
			break;
		case dtAlert:
			if (Dialogs->AlertProc)
				Dialogs->AlertProc(Dialogs->HandlerClass);
			break;
		default:
			break;
	}
}

void CDialogs::PwdKeyPress(CDialogs *Dialogs, int &Key)
{
	if (Key == KEY_ENTER) {
		Key = KEY_NONE;
		Dialogs->CloseBtnClick(Dialogs);
	}
}

void CDialogs::ShowMessageBox(const char *Title, const char *Text, const char *ButtonCaption)
{
	int ClientWidth;
	int ButtonLeft;
	int ScrnWidth, ScrnHeight;
	int BoxLeft, BoxTop;
	int TextLeft;

	ClientWidth = Graph->GetTextWidth(Text,STYLE_REGULAR) + 32;
	if (ClientWidth < 200) {
		TextLeft = (232 - ClientWidth) >> 1;
		ClientWidth = 200;
	}
	else
		TextLeft = 16;
	Graph->GetModeMetrics(ScrnWidth,ScrnHeight);
	BoxLeft = (ScrnWidth - ClientWidth - 4) >> 1;
	BoxTop = (ScrnHeight - ClientHeight - 24) >> 1;
	ButtonLeft = (ClientWidth - 75) >> 1;
	if (Parent)
		Parent->Disable();
	Form->SetVisible(FALSE);
	Form->SetCaption(Title);
	Form->SetPosition(BoxLeft,BoxTop);
	Form->SetMetrics(ClientWidth + 4,ClientHeight + 32);
	BodyEdit->Hide();
	BodyText->Show();
	BodyText->SetCaption(Text);
	BodyText->SetPosition(TextLeft,24);
	CloseBtn->SetPosition(ButtonLeft,64);
	CloseBtn->SetCaption(ButtonCaption);
	Form->FocusControl(CloseBtn);
	Form->Show();
}




