/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#ifndef __messagebox__
#define __messagebox__

#include <gui.h>

typedef void (*TPwdProc)(void *HandlerClass, const char *Password);
typedef void (*TAlertProc)(void *HandlerClass);


class CDialogs {
public:
	CDialogs(CScreen *ScreenToUse, CForm *Parent, void *HandlerClass);
	~CDialogs();

	void ShowMessageDialog(const char *Title, const char *Text);
	void ShowPasswordDialog(const char *Title, TPwdProc PwdProcToUse);
	void ShowAlertDialog(const char *Title, const char *Text, 
		const char *ButtonCaption, TAlertProc AlertProcToUse);
private:
	typedef enum { dtMessage, dtPassword, dtAlert } TDialogType;

	CScreen *Screen;

	CForm *Parent;
	void *HandlerClass;

	TPwdProc PwdProc;
	TAlertProc AlertProc;

	TDialogType DialogType;

	// control on the dialogs
	CForm *Form;
	CLabel *BodyText;
	CEdit *BodyEdit;
	CButton *CloseBtn;

	static void CloseBtnClick(CDialogs *Dialogs);
	static void PwdKeyPress(CDialogs *Dialogs, int &Key);

	void ShowMessageBox(const char *Title, const char *Text, const char *ButtonCaption);

};

#endif