/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#ifndef __items__
#define __items__

#include <newdefs.h>
#include <lastconf.h>
#include <ptab.h>
#include <fs.h>
#include <key.h>
#include <fatfix.h>

#define ITEM_IGNORE 0
#define ITEM_HIDE   1
#define ITEM_REVEAL 2

typedef struct {
	char ItemName[48];
	unsigned char Keys[16];
	short Activate;
	unsigned long Password;
	short PartIndex;
	short Valid;
	unsigned short Hotkey;
	char FixDriveNum;
	char Reserved;
	char HidingItems[92];
} TBootItem;

typedef struct {
	short BootItemCount;
	short PartCount;
	short InstantBoot;
	short Timeout;
	short Reserved[4];
	TBootItem BootItems[24];
} TItemFile;

class CBootItems {
	public:
		CBootItems(CFileSystem *FileSystemToUse, CPartList *PartListToUse, CKeyboard *KeyboardToUse);
		~CBootItems();
		int GetCount();
		TBootItem *GetBootItem(int Index);
		void Add(TBootItem &NewItem);
		void Insert(int Index,TBootItem &NewItem);
		void Remove(int Index);
		void Clear();
		void MoveUp(int Index);
		void MoveDown(int Index);
		void Save();
		void Reset();

		int GetInstantBoot();
		void SetInstantBoot(int Index);
		int GetTimeout();
		void SetTimeout(int Timeout);
		int GetPartCount();

		void DeinitBoot();
		int InitBoot(int BootIndex);

		int CanFix(const TBootItem *Entry);

	private:
		CFATFix *FATFix;
		TItemFile *ItemFile;
		void ItemsMoveUp(int Index);
		void ItemsMoveDown(int Index);

		void UpdateBootItems(const short *RelocTable);
      void ValidateHidingItems();

		void StoreKeys(const unsigned char *Keys);

		CLastConfig *LastConfig;
		CFileSystem *FileSystem;
		CPartList *PartList;
		CKeyboard *Keyboard;

};



#endif