/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <loader.h>
#include <about.h>
#include <setup.h>
#include <prefer.h>


#include <text.h>
#include <string.h>

void printf(const char *format, ...)
{
	unsigned short *argl;
	char buf[32];

	for (argl = &((unsigned short *)&format)[2]; *format; ++format)
		switch (*format) {
			case '\n':
				puts("\r\n");
				break;
			case '%':
				switch (*++format) {
					case 'c':
						putch(*argl++);
						break;
					case 'd':
						itoa(*argl++,buf,DEC);
						puts(buf);
						break;
					case 'x':
						itoa(*argl++,buf,HEX);
						puts(buf);
						break;
					case 's':
						puts((const char *)*(long *)argl);
						argl += 2;
						break;
					case 'l':
						if (format[1] == 'd') {
							++format;
							itoa(*(unsigned long *)argl++,buf,DEC);
							puts(buf);
							break;
						}
						if (format[1] == 'x') {
							++format;
							itoa(*(unsigned long *)argl++,buf,HEX);
							puts(buf);
							break;
						}
					default:
						putch('%');
						--format;
						break;
				}
				break;
			default:
				putch(*format);
				break;
		}
}



extern char *LogoBitmap;

static const char *StrVersion = "XOSL version 1.1.0a";
static const char *StrCopyright = "Copyright  1999, Geurt Vos";
static const char *StrURL = "http://www.xosl.org";
static const char *StrEndTimer = "Press Escape to abort timer...";

static const char *StrLabel5 = "Choose OS";

int CLoader::EnterSetup;

CLoader::CLoader()
{
}

CLoader::~CLoader()
{
}

void CLoader::Initialize(CPreference *PreferenceToUse, CSetup *SetupToUse,
						 CAbout *AboutToUse, CScreen *ScreenToUse,
						 CBootItems *BootItemsToUse, CXOSLData *XOSLDataToUse)

{
	Preference = PreferenceToUse;
	Setup = SetupToUse;
	About = AboutToUse;

	Screen = ScreenToUse;
	BootItems = BootItemsToUse;
	XOSLData = XOSLDataToUse;

	BootBtnPressed = FALSE;
	if ((BootItemIndex = BootItems->GetInstantBoot()) == -1)
		BootItemIndex = 0;

	CreateControls();
	InstallControls();
	CreateBootList(BootItemIndex);
	InitInstantBoot();
	InitializeControls();
	Dialogs = new CDialogs(Screen,Form,this);

}

void CLoader::Show()
{
	if (!Form->Visible) {
		Form->Show();
		Form->FocusControl(BootBtn);
	}
	else
		Screen->SetFocusWnd(Form);
}

void CLoader::Hide()
{
	if (Form->Visible)
		Form->Hide();
}

int CLoader::IsVisible()
{
	return Form->Visible && Form->IsEnabled();
}

void CLoader::Focus()
{
	Screen->SetFocusWnd(Form);
}

int CLoader::GotFocus()
{
	return Screen->GetFocusWnd() == Form;
}

void CLoader::CreateControls()
{
	int Width;

	Form = new CForm("Extended Operating System Loader",FORM_NORMAL,TRUE,100,100,461,328,FALSE);
	Image = new CImage(LogoBitmap,218,146,TRUE,8,16,220,148,TRUE);
	Width = Graph->GetTextWidth(StrVersion,STYLE_REGULAR);
	Label1 = new CLabel(StrVersion,STYLE_REGULAR,TRUE,17,8 + (218 - Width >> 1),168,Width,TRUE);
	Width = Graph->GetTextWidth(StrCopyright,STYLE_REGULAR);
	Label2 = new CLabel(StrCopyright,STYLE_REGULAR,TRUE,17,8 + (218 - Width >> 1),184,Width,TRUE);
	Width = Graph->GetTextWidth(StrURL,STYLE_REGULAR);
	Label3 = new CLabel(StrURL,STYLE_REGULAR,TRUE,22,8 + (218 - Width >> 1),200,Width,TRUE);
	Width = Graph->GetTextWidth(StrEndTimer,STYLE_REGULAR);
	Label4 = new CLabel(StrEndTimer,STYLE_REGULAR,TRUE,17,8 + (218 - Width >> 1),240,Width,FALSE);
	Label5 = new CLabel(StrLabel5,STYLE_REGULAR,FALSE,17,248,9,TRUE);
	BootLabel = new CLabel("",STYLE_REGULAR,TRUE,17,240,264,FALSE);
	Bevel = new CBevel(BEVEL_FRAME,TRUE,240,16,209,193,TRUE);
	BootBtn = new CButton("Boot",307,224,75,25,TRUE,this);
	SetupBtn = new CButton("Setup",8,272,65,25,TRUE,this);
	PreferBtn = new CButton("Preference",80,272,75,25,TRUE,this);
	AboutBtn = new CButton("About",160,272,65,25,TRUE,this);
	ProgressBar = new CProgressBar(0,0,0,240,280,209,17,FALSE);
	BootItemsList = new CRadioGroup(256,32,173,24 * 7,TRUE,this);
	ScrollBar = new CScrollBar(0,0,0,TRUE,432,16,193,FALSE,BootItemsList);
}


void CLoader::InitializeControls()
{
	int Index;

	BootBtn->SetFontStyle(STYLE_BOLD);
	BootBtn->OnClick((TWndOnClick)BootBtnClick);
	AboutBtn->OnClick((TWndOnClick)AboutBtnClick);
	SetupBtn->OnClick((TWndOnClick)SetupBtnClick);
	PreferBtn->OnClick((TWndOnClick)PreferBtnClick);

	BootBtn->OnKeyPress((TWndOnKeyPress)ButtonKeyPress);
	AboutBtn->OnKeyPress((TWndOnKeyPress)ButtonKeyPress);
	SetupBtn->OnKeyPress((TWndOnKeyPress)ButtonKeyPress);
	PreferBtn->OnKeyPress((TWndOnKeyPress)ButtonKeyPress);

	BootItemsList->OnSelect((TListBoxSelect)BootItemsSelect);
	BootItemsList->OnKeyPress((TWndOnKeyPress)BootItemsKeyPress);
	BootItemsList->SetScrollBar(ScrollBar);
}

void CLoader::InstallControls()
{
	int Index;

	Form->AddControl(Bevel);
	Form->AddControl(Image);
	Form->AddControl(Label1);
	Form->AddControl(Label2);
	Form->AddControl(Label3);
	Form->AddControl(Label4);
	Form->AddControl(Label5);
	Form->AddControl(BootLabel);
	Form->AddControl(BootBtn);
	Form->AddControl(SetupBtn);
	Form->AddControl(PreferBtn);
	Form->AddControl(AboutBtn);
	Form->AddControl(ProgressBar);
	Form->AddControl(BootItemsList);
	Form->AddControl(ScrollBar);
	Screen->AddWindow(Form);
}


void CLoader::CreateBootList(int StartIndex)
{
	int Count;
	int Index;
	int Max;
	static char *StrIPart = "1. ";

	Count = BootItems->GetCount();
	if (StartIndex >= Count)
		StartIndex = 0;
	BootItemIndex = StartIndex;
	if (!Count || !BootItems->GetBootItem(StartIndex)->Valid)
		BootBtn->Disable();
	else
		BootBtn->Enable();
	BootItemsList->Clear();
	if (!XOSLData->GetMiscPref()->DisplayIndex)
		for (Index = 0; Index < Count; ++Index)
			BootItemsList->AddButton(BootItems->GetBootItem(Index)->ItemName);
	else
		for (Index = 0; Index < Count; ++Index) {
			if (Index < 9)
				*StrIPart = Index + '1';
			else
				*StrIPart = Index - 9 + 'A';

			CString IndexStr(StrIPart);
			IndexStr += BootItems->GetBootItem(Index)->ItemName;
			BootItemsList->AddButton(IndexStr);
		}
	if (Count && StartIndex != BootItemsList->GetChecked())
		BootItemsList->SetChecked(StartIndex);
	else
		BootItemsList->Refresh();
	BootItemsList->SetEnabled(Count != 0);
}

void CLoader::InitInstantBoot()
{
	int Timeout;

	if (BootItems->GetInstantBoot() != -1) {
		Timeout = BootItems->GetTimeout();
		ProgressBar->SetMax(Timeout << 4);
		ProgressBar->SetVisible(TRUE);
		BootLabel->SetVisible(TRUE);
		Label4->SetVisible(TRUE);
	}
}

void CLoader::AbortInstantBoot()
{
	ProgressBar->Hide();
	BootLabel->Hide();
	Label4->Hide();
}

void CLoader::ShowBootError(const char *ErrMsg)
{
	Dialogs->ShowMessageDialog("Boot error",ErrMsg);
}

void CLoader::SetTimeoutLabelCaption(const char *BootStr)
{
	BootLabel->SetCaption(BootStr);
}

void CLoader::SetTimeoutProgress(int Progress)
{
	ProgressBar->SetProgress(Progress);
}	

void CLoader::CenterWindow()
{
	int Width, Height;

	Graph->GetModeMetrics(Width,Height);
	Form->SetPosition((Width - 461) >> 1,(Height - 328) >> 1);
}

void CLoader::RealignText()
{
	int Left,Top;

	Label1->SetVisible(FALSE);
	Label2->SetVisible(FALSE);
	Label3->SetVisible(FALSE);
	Label5->SetVisible(FALSE);

	Label1->GetPosition(Left,Top);
	Left = 8 + (218 - Graph->GetTextWidth(StrVersion,STYLE_REGULAR) >> 1);
	Label1->SetPosition(Left,Top);

	Label2->GetPosition(Left,Top);
	Left = 8 + (218 - Graph->GetTextWidth(StrCopyright,STYLE_REGULAR) >> 1);
	Label2->SetPosition(Left,Top);

	Label3->GetPosition(Left,Top);
	Left = 8 + (218 - Graph->GetTextWidth(StrURL,STYLE_REGULAR) >> 1);
	Label3->SetPosition(Left,Top);

	Label4->GetPosition(Left,Top);
	Left = 8 + (218 - Graph->GetTextWidth(StrEndTimer,STYLE_REGULAR) >> 1);
	Label4->SetPosition(Left,Top);

	Label5->SetCaption(StrLabel5);

	Label1->SetVisible(TRUE);
	Label2->SetVisible(TRUE);
	Label3->SetVisible(TRUE);
	Label5->SetVisible(TRUE);

}


int CLoader::GetBootItemIndex()
{
	return BootItemIndex;
}

int CLoader::CanBoot()
{
	return BootBtnPressed;
}

void CLoader::CancelBoot()
{
	BootBtnPressed = FALSE;
}


//////
void CLoader::BootBtnClick(CLoader *Loader)
{
	int ItemIndex;
	const TBootItem *Item;

	ItemIndex = Loader->BootItemsList->GetChecked();
	Item = Loader->BootItems->GetBootItem(ItemIndex);
	if (Item && Item->Valid)
		if (Loader->BootItems->GetBootItem(Loader->BootItemIndex)->Password)
			Loader->Dialogs->ShowPasswordDialog("Enter password",(TPwdProc)BootPwdProc);
		else
			Loader->BootBtnPressed = TRUE;
}

void CLoader::AboutBtnClick(CLoader *Loader)
{
	Loader->About->Show();
}

void CLoader::SetupBtnClick(CLoader *Loader)
{
	if (!Loader->Setup->IsVisible() && Loader->XOSLData->GetPassword()) {
		EnterSetup = TRUE;
		Loader->Dialogs->ShowPasswordDialog("Setup password",(TPwdProc)PwdProc);
	}
	else
		Loader->Setup->Show();
}

void CLoader::PreferBtnClick(CLoader *Loader)
{
	if (!Loader->Preference->IsVisible() && Loader->XOSLData->GetPassword()) {
		EnterSetup = FALSE;
		Loader->Dialogs->ShowPasswordDialog("Preference password",(TPwdProc)PwdProc);
	}
	else
		Loader->Preference->Show();
}

void CLoader::PwdProc(CLoader *Loader, const char *Password)
{
	unsigned long Code;

	Code = EncodePassword(Password);
	if (Loader->XOSLData->GetPassword() == Code)
		if (EnterSetup)
			Loader->Setup->Show();
		else
			Loader->Preference->Show();
	else
		if (EnterSetup)
			Loader->Dialogs->ShowMessageDialog("Setup password","Invalid Password");
		else
			Loader->Dialogs->ShowMessageDialog("Preference password","Invalid Password");
}

void CLoader::BootItemsSelect(CLoader *Loader, int ItemIndex)
{
	Loader->BootItemIndex = ItemIndex;
	Loader->BootBtn->SetEnabled(Loader->BootItems->GetBootItem(ItemIndex)->Valid);
}

void CLoader::BootPwdProc(CLoader *Loader, const char *Password)
{
	unsigned long Code;

	Code = EncodePassword(Password);
	if (Loader->BootItems->GetBootItem(Loader->BootItemIndex)->Password == Code)
		Loader->BootBtnPressed = TRUE;
	else
		Loader->Dialogs->ShowMessageDialog("Password","Invalid password.");
}

void CLoader::BootItemsKeyPress(CLoader *Loader, int &Key)
{
	if (Key == KEY_ENTER)
		BootBtnClick(Loader);
	else
		if (Key != KEY_UP && Key != KEY_DOWN)
			ButtonKeyPress(Loader,Key);
}

void CLoader::ButtonKeyPress(CLoader *Loader, int &Key)
{
	int Index, Count;
	TBootItem *Entry;
	int lKey;

	if (!Loader->BootItems->GetCount() || !Key)
		return;
	Count = Loader->BootItems->GetCount();

	// check for hotkeys
	for (Index = 0; Index < Count; ++Index) {
		Entry = Loader->BootItems->GetBootItem(Index);
		if (Entry->Hotkey == Key) {
			Loader->BootItemsList->SetChecked(Index);
			// always instantly boot
			Loader->BootBtnClick(Loader);
			Key = 0;
			return;
		}
	}
	// check for 1-9,a-n
	lKey = tolower(Key & 0xff);
	if (Key == KEY_UP || Key == KEY_DOWN) {
		Loader->Form->FocusControl(Loader->BootItemsList);
		Loader->BootItemsList->KeyPress(Key);
	}
	else
		if ( (lKey >= '1' && lKey <= '9') ||
			  (lKey >= 'a' && lKey <= 'n') ) {

			if (lKey >= '1' && lKey <= '9')
				Index = lKey - '1';
			else
				Index = lKey - 'a' + 9;
			if (Index < Count) {
				Loader->BootItemsList->SetChecked(Index);
				if (Loader->XOSLData->GetMiscPref()->BootOnSelect)
					Loader->BootBtnClick(Loader);
				else
					Loader->Form->FocusControl(Loader->BootItemsList);
			}
		}
		else
			return;
	Key = 0;
}
