/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#ifndef __loader__
#define __loader__

#include <gui.h>
#include <graph.h>
#include <items.h>
#include <xosldata.h>
#include <encpwd.h>
#include <dialogs.h>
#include <key.h>
#include <ctype.h>


class CAbout;
class CPreference;
class CSetup;

class CLoader {
	public:
		CLoader();
		~CLoader();

		void Initialize(CPreference *PreferenceToUse, CSetup *SetupToUse,
			CAbout *AboutToUse, CScreen *ScreenToUse,
			CBootItems *BootItemsToUse, CXOSLData *XOSLDataToUse);

		void Show();
		void Hide();
		int IsVisible();
		void Focus();
		int GotFocus();
		int CanBoot();
		int GetBootItemIndex();
		void CancelBoot();

		void CenterWindow();

		void ShowBootError(const char *ErrMsg);
		void AbortInstantBoot();
		void SetTimeoutLabelCaption(const char *BootStr);
		void SetTimeoutProgress(int Progress);

		void CreateBootList(int StartIndex);
		void RealignText();

	private:
		CPreference *Preference;
		CSetup *Setup;
		CAbout *About;

		CScreen *Screen;
		CBootItems *BootItems;
		CXOSLData *XOSLData;

		CDialogs *Dialogs;

	private:

		static int EnterSetup;

		static void BootBtnClick(CLoader *Loader);
		static void AboutBtnClick(CLoader *Loader);
		static void SetupBtnClick(CLoader *Loader);
		static void PreferBtnClick(CLoader *Loader);
		static void PwdProc(CLoader *Loader, const char *Password);
		static void BootPwdProc(CLoader *Loader, const char *Password);
		static void ButtonKeyPress(CLoader *Loader, int &Key);
		static void BootListPress(CLoader *Loader, int &Key);
		static void BootItemsSelect(CLoader *Loader, int ItemIndex);
		static void BootItemsKeyPress(CLoader *Loader, int &Key);


	private:

		CForm *Form;
		CImage *Image;
		CLabel *Label1;
		CLabel *Label2;
		CLabel *Label3;
		CLabel *Label4;
		CLabel *Label5;
		CLabel *BootLabel;
		CBevel *Bevel;
		CScrollBar *ScrollBar;
		CButton *BootBtn;
		CButton *SetupBtn;
		CButton *PreferBtn;
		CButton *AboutBtn;
		CProgressBar *ProgressBar;
		CRadioGroup *BootItemsList;

		int BootItemIndex;
		int StartIndex;
		int BootBtnPressed;



	private:
		void CreateControls();
		void InitializeControls();
		void InstallControls();
		void InitInstantBoot();

};



#endif
