/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <prefer.h>

#include <text.h>
extern void printf(const char *fmt,...);

#define TAB_GRAPHICS 0
#define TAB_COLOR    1
#define TAB_MOUSE    2
#define TAB_PASSWORD 3
#define TAB_MISC     4


extern void *AdditionalFont;
extern void *DefaultFont;

static const char *ColorSchemes[16] = {
	"Brick","Desert","Eggplant","Lilac","Maple","Marine","Plum",
	"Pumpkin","Rainy day","Red, white & blue","Rose","Slate",
	"Spruce","Strom","Teal","XOSL standard"
};


static const char *StrGrScrnArea = "Screen area";
static const char *StrGrPersonal = "Personal";
static const char *StrGrVisual = "Visual effects";
static const char *StrGrEnhance = "Enhancement";

static const char *StrClColorScheme = "Color scheme";
static const char *StrClAdjustment = "Color adjustment";
static const char *StrClFadeColor = "Fade-in color";
static const char *StrMoTypeLabel = "Mouse type";
static const char *StrMoSpeedLabel = "Mouse speed";
static const char *StrMoSamplingLabel = "PS/2 sampling rate";
static const char *StrPwLabel = "Main password";
static const char *StrMiscKeys = "Misc keys";
static const char *StrHandling = "Boot handling";
static const char *StrMiInstBoot = "Instant boot";


int CPreference::NoUpdates = 1;


CPreference::CPreference(CLoader *LoaderToUse, CSetup *SetupToUse,
						 CAbout *AboutToUse, CScreen *ScreenToUse, 
						 CXOSLData *XOSLDataToUse, CKeyboard *KeyboardToUse,
						 CMouse *MouseToUse)
{
	Loader = LoaderToUse;
	Setup = SetupToUse;
	About = AboutToUse;
	Screen = ScreenToUse;
	XOSLData = XOSLDataToUse;
	Keyboard = KeyboardToUse;
	Mouse = MouseToUse;

	IgnoreKey = 0;
	CreateControls();
	InitializeControls();
	InstallControls();
	NoUpdates = FALSE;

	Dialogs = new CDialogs(ScreenToUse,Form,this);
}

CPreference::~CPreference()
{
}

void CPreference::Show()
{
	if (!Form->Visible) {
		Form->Show();
		Form->FocusControl(CloseBtn);
	}
	else
		Screen->SetFocusWnd(Form);
}

void CPreference::Hide()
{
	if (Form->Visible)
		Form->Hide();
}

int CPreference::IsVisible()
{
	return Form->Visible && Form->IsEnabled();
}

int CPreference::GotFocus()
{
	return Screen->GetFocusWnd() == Form;
}

int CPreference::GetIgnoreNextKey()
{
	if (IgnoreKey) {
		IgnoreKey = 0;
		return 1;
	}
	return 0;
}

void CPreference::SetIgnoreNextKey()
{
	IgnoreKey = 1;
}

void CPreference::CreateControls()
{
	// main controls
	Form = new CForm("XOSL Preference",FORM_NORMAL,TRUE,200,200,478,385,FALSE);
	TabBevel = new CBevel(BEVEL_BOX,FALSE,8,8,457,290,TRUE);
	TabControl = new CTabControl(5,8,297,350,TRUE,this);
	ResetBtn = new CButton("Reset",205,328,75,25,TRUE,this);
	SaveBtn = new CButton("Save",292,328,75,25,TRUE,this);
	CloseBtn = new CButton("Close",391,328,75,25,TRUE,this);
	// controls in graphics tab page
	GrModeBevel = new CBevel(BEVEL_FRAME,TRUE,20,38,161,97,TRUE);
	GrVideoMode = new CLabel(StrGrScrnArea,STYLE_REGULAR,FALSE,17,28,31,TRUE);
	GrModeSlider = new CTrackBar(0,Graph->GetModeCount() - 1,0,36,54,129,TRUE,this);
	GrModeValue = new CLabel("",STYLE_REGULAR,TRUE,17,70,70,TRUE);
	GrTest = new CButton("Test",31,94,67,25,TRUE,this);
	GrApply = new CButton("Apply",103,94,67,25,TRUE,this);

	GrPersonalBevel = new CBevel(BEVEL_FRAME,TRUE,20,150,161,73,TRUE);
	GrPersonalLabel = new CLabel(StrGrPersonal,STYLE_REGULAR,FALSE,17,28,143,TRUE);
	GrFadeIn = new CCheckBox("Disable fade-in",FALSE,44,166,TRUE,this);
	GrFadeOut = new CCheckBox("Disable fade-out",FALSE,44,190,TRUE,this);

	GrVisualBevel = new CBevel(BEVEL_FRAME,TRUE,188,38,265,121,TRUE);
	GrVisualLabel = new CLabel(StrGrVisual,STYLE_REGULAR,FALSE,17,196,31,TRUE);
	GrSWCWD = new CCheckBox("Show window contents while dragging",FALSE,212,54,TRUE,this);
	GrFont = new CCheckBox("Use 9 point font",FALSE,212,78,TRUE,this);
	GrInvCursor = new CCheckBox("Invert mouse cursor color",FALSE,212,102,TRUE,this);
	GrWallpaper = new CCheckBox("Use wallpaper",FALSE,212,126,TRUE,this);

	GrEnhanceBevel = new CBevel(BEVEL_FRAME,TRUE,188,174,265,49,TRUE);
	GrEnhanceLabel = new CLabel(StrGrEnhance,STYLE_REGULAR,FALSE,17,196,167,TRUE);
	GrULFB = new CCheckBox("Use linear frame buffer",FALSE,212,190,TRUE,this);


	// controls in color tab page
	ClSchemeGroup = new CBevel(BEVEL_FRAME,TRUE,24,32,177,217,FALSE);
	ClColorScheme = new CLabel(StrClColorScheme,STYLE_REGULAR,FALSE,17,32,25,FALSE);
	ClListBox = new CListBox(1,FALSE,48,56,129,169,FALSE,this);
	ClScrollBar = new CScrollBar(0,0,0,FALSE,158,57,167,FALSE,ClListBox);
	ClAdjustGroup = new CBevel(BEVEL_FRAME,TRUE,216,32,233,137,FALSE);
	ClAdjustment = new CLabel(StrClAdjustment,STYLE_REGULAR,FALSE,17,224,25,FALSE);
	ClHueLabel = new CLabel("Hue:",STYLE_REGULAR,TRUE,17,232,57,FALSE);
	ClSatLabel = new CLabel("Saturation:",STYLE_REGULAR,TRUE,17,232,89,FALSE);
	ClLumLabel = new CLabel("Luminance:",STYLE_REGULAR,TRUE,17,232,121,FALSE);
	ClHueValue = new CLabel("0",STYLE_REGULAR,TRUE,17,408,72,39,FALSE);
	ClSatValue = new CLabel("0%",STYLE_REGULAR,TRUE,17,408,104,39,FALSE);
	ClLumValue = new CLabel("0%",STYLE_REGULAR,TRUE,17,408,136,39,FALSE);
	ClHueSlider = new CTrackBar(-63,63,0,232,72,169,FALSE,this);
	ClSatSlider = new CTrackBar(-63,63,0,232,104,169,FALSE,this);
	ClLumSlider = new CTrackBar(-51,50,0,232,136,169,FALSE,this);

	ClFadeGroup = new CBevel(BEVEL_FRAME,TRUE,216,184,233,97,FALSE);
	ClFadeColor = new CLabel(StrClFadeColor,STYLE_REGULAR,FALSE,17,224,177,FALSE);
	ClFadeIntLabel = new CLabel("Intensity:",STYLE_REGULAR,TRUE,17,232,205,FALSE);
	ClFadeIntSlider = new CTrackBar(0,100,100,232,220,169,FALSE,this);
	ClFadeIntValue = new CLabel("100%",STYLE_REGULAR,TRUE,17,408,220,39,FALSE);
	ClFadeTestBtn = new CButton("Test",295,244,75,25,FALSE,this);

	// controls in mouse tab page
	MoTypeGroup = new CBevel(BEVEL_FRAME,TRUE,24,32,161,217,FALSE);
	MoTypeLabel = new CLabel(StrMoTypeLabel,STYLE_REGULAR,FALSE,17,32,25,FALSE);
	MoMouseGroup = new CRadioGroup(56,48,120,144,FALSE,this);
	MoApply = new CButton("Apply",67,204,75,25,FALSE,this);

	MoSpeedGroup = new CBevel(BEVEL_FRAME,TRUE,200,32,241,41,FALSE);
	MoSpeedLabel = new CLabel(StrMoSpeedLabel,STYLE_REGULAR,FALSE,17,208,25,FALSE);
	MoSpeedSlider = new CTrackBar(0,8,7,228,48,201,FALSE,this);
	MoSamplingGroup = new CBevel(BEVEL_FRAME,TRUE,200,96,241,41,FALSE);
	MoSamplingLabel = new CLabel(StrMoSamplingLabel,STYLE_REGULAR,FALSE,17,208,89,FALSE);
	MoSamplingSlider = new CTrackBar(0,6,0,228,112,201,FALSE,this);
	// controls in password tab page
	PwGroup = new CBevel(BEVEL_FRAME,TRUE,132,32,201,217,FALSE);
	PwLabel = new CLabel(StrPwLabel,STYLE_REGULAR,FALSE,17,140,25,FALSE);
	PwOldLabel = new CLabel("Old password:",STYLE_REGULAR,TRUE,17,172,56,FALSE);
	PwNewLabel = new CLabel("New password:",STYLE_REGULAR,TRUE,17,172,104,FALSE);
	PwReLabel = new CLabel("Re-enter password:",STYLE_REGULAR,TRUE,17,172,152,FALSE);
	PwOldEdit = new CEdit("",32,TRUE,FALSE,172,72,121,FALSE,this);
	PwNewEdit = new CEdit("",32,TRUE,FALSE,172,120,121,FALSE,this);
	PwReEdit = new CEdit("",32,TRUE,FALSE,172,168,121,FALSE,this);
	PwApplyBtn = new CButton("Apply",195,208,75,25,FALSE,this);

	// controls in misc tab page
	MiKeysBevel = new CBevel(BEVEL_FRAME,TRUE,44,34,161,249,FALSE);
	MiKeysLabel = new CLabel(StrMiscKeys,STYLE_REGULAR,FALSE,17,52,27,FALSE);
	MiCycleLabel = new CLabel("Cycle windows",STYLE_REGULAR,TRUE,17,68,50,FALSE);
	MiCycleEdit = new CEdit("",10,FALSE,FALSE,68,66,113,FALSE,this);
	MiRestartLabel = new CLabel("Restart XOSL",STYLE_REGULAR,TRUE,17,68,106,FALSE);
	MiRestartEdit = new CEdit("",10,FALSE,FALSE,68,122,113,FALSE,this);
	MiRebootLabel = new CLabel("Cold reboot",STYLE_REGULAR,TRUE,17,68,154,FALSE);
	MiRebootEdit = new CEdit("",10,FALSE,FALSE,68,170,113,FALSE,this);
	MiShutdownLabel = new CLabel("Shutdown",STYLE_REGULAR,TRUE,17,68,202,FALSE);
	MiShutdownEdit = new CEdit("",10,FALSE,FALSE,68,218,113,FALSE,this);
	MiPowerOff = new CCheckBox("Soft power off",FALSE,76,250,FALSE,this);

	MiHandlingBevel = new CBevel(BEVEL_FRAME,TRUE,212,34,217,133,FALSE);
	MiHandlingLabel = new CLabel(StrHandling,STYLE_REGULAR,FALSE,17,220,27,FALSE);
	MiDisplayIndex = new CCheckBox("Display boot item index",FALSE,236,58,FALSE,this);
	MiBootSelect = new CCheckBox("Boot on key press",FALSE,236,82,FALSE,this);
	MiActiveAllow = new CCheckBox("Allow active partition per HD",FALSE,236,106,FALSE,this);
	MiAutoReveal = new CCheckBox("Reveal partitions on start-up",FALSE,236,130,FALSE,this);

	MiInstBootBevel = new CBevel(BEVEL_FRAME,TRUE,212,178,217,85,FALSE);
	MiInstBootLabel = new CLabel(StrMiInstBoot,STYLE_REGULAR,FALSE,17,220,171,FALSE);
	MiIgnoreClick = new CCheckBox("Ignore mouse click",FALSE,236,202,FALSE,this);
	MiSetInstBoot = new CCheckBox("Set item as instant boot",FALSE,236,226,FALSE,this);

}

void CPreference::InitializeControls()
{
	int Index;

	TabControl->SetTabCaption(0,"Graphics");
	TabControl->SetTabCaption(1,"Color");
	TabControl->SetTabCaption(2,"Mouse");
	TabControl->SetTabCaption(3,"Password");
	TabControl->SetTabCaption(4,"Misc");
	TabControl->SetTabPage(TabBevel);
	CloseBtn->OnClick((TWndOnClick)CloseBtnClick);
	ResetBtn->OnClick((TWndOnClick)ResetBtnClick);
	SaveBtn->OnClick((TWndOnClick)SaveBtnClick);
	// graphics tab page
	if (!Graph->LinearSupport())
		GrULFB->Disable();
	TabControl->AddControl(0,GrModeBevel);
	TabControl->AddControl(0,GrVideoMode);
	TabControl->AddControl(0,GrModeSlider);
	TabControl->AddControl(0,GrModeValue);
	TabControl->AddControl(0,GrTest);
	TabControl->AddControl(0,GrApply);

	TabControl->AddControl(0,GrPersonalBevel);
	TabControl->AddControl(0,GrPersonalLabel);
	TabControl->AddControl(0,GrFadeIn);
	TabControl->AddControl(0,GrFadeOut);

	TabControl->AddControl(0,GrVisualBevel);
	TabControl->AddControl(0,GrVisualLabel);
	TabControl->AddControl(0,GrSWCWD);
	TabControl->AddControl(0,GrFont);
	TabControl->AddControl(0,GrInvCursor);
	TabControl->AddControl(0,GrWallpaper);

	TabControl->AddControl(0,GrEnhanceBevel);
	TabControl->AddControl(0,GrEnhanceLabel);
	TabControl->AddControl(0,GrULFB);

	GrModeSlider->OnChange((TTrackBarChange)GrModeSliderChange);
	GrSWCWD->OnChange((TCheckBoxChange)GrSWCWDChange);
	GrInvCursor->OnChange((TCheckBoxChange)GrInvCursorChange);
	GrFont->OnChange((TCheckBoxChange)GrFontChange);
	GrWallpaper->OnChange((TCheckBoxChange)GrWallpaperChange);

	// color tab page

	TabControl->AddControl(1,ClSchemeGroup);
	TabControl->AddControl(1,ClColorScheme);
	TabControl->AddControl(1,ClListBox);
	TabControl->AddControl(1,ClScrollBar);
	TabControl->AddControl(1,ClAdjustGroup);
	TabControl->AddControl(1,ClAdjustment);
	TabControl->AddControl(1,ClHueLabel);
	TabControl->AddControl(1,ClSatLabel);
	TabControl->AddControl(1,ClLumLabel);
	TabControl->AddControl(1,ClHueValue);
	TabControl->AddControl(1,ClSatValue);
	TabControl->AddControl(1,ClLumValue);
	TabControl->AddControl(1,ClHueSlider);
	TabControl->AddControl(1,ClSatSlider);
	TabControl->AddControl(1,ClLumSlider);
	TabControl->AddControl(1,ClFadeGroup);
	TabControl->AddControl(1,ClFadeColor);
	TabControl->AddControl(1,ClFadeIntSlider);
	TabControl->AddControl(1,ClFadeIntLabel);
	TabControl->AddControl(1,ClFadeIntValue);
	TabControl->AddControl(1,ClFadeTestBtn);

	ClListBox->SetColumn(0,108,"");
	for (Index = 0; Index < 16; ++Index)
		ClListBox->AddItem(ClListBox->AddRow(),0,ColorSchemes[Index]);
	ClListBox->SetScrollBar(ClScrollBar);
	ClListBox->OnSelect((TListBoxSelect)ColorSchemeSelect);
	ClHueSlider->OnChange((TTrackBarChange)ChangeHue);
	ClSatSlider->OnChange((TTrackBarChange)ChangeSat);
	ClLumSlider->OnChange((TTrackBarChange)ChangeLum);
	TabControl->OnChange((TTabChange)TabChange);

	ClFadeIntSlider->OnChange((TTrackBarChange)FadeIntChange);
	ClFadeTestBtn->OnClick((TWndOnClick)FadeTestClick);


	// mouse tab page
	TabControl->AddControl(2,MoTypeGroup);
	TabControl->AddControl(2,MoTypeLabel);
	TabControl->AddControl(2,MoMouseGroup);
	TabControl->AddControl(2,MoSpeedGroup);
	TabControl->AddControl(2,MoSpeedLabel);
	TabControl->AddControl(2,MoSpeedSlider);
	TabControl->AddControl(2,MoSamplingGroup);
	TabControl->AddControl(2,MoSamplingLabel);
	TabControl->AddControl(2,MoSamplingSlider);
	TabControl->AddControl(2,MoApply);

	MoMouseGroup->AddButton("Serial COM1");
	MoMouseGroup->AddButton("Serial COM2");
	MoMouseGroup->AddButton("Serial COM3");
	MoMouseGroup->AddButton("Serial COM4");
	MoMouseGroup->AddButton("PS/2");
	MoMouseGroup->AddButton("None");
	// password tab page
	TabControl->AddControl(3,PwGroup);
	TabControl->AddControl(3,PwLabel);
	TabControl->AddControl(3,PwOldLabel);
	TabControl->AddControl(3,PwNewLabel);
	TabControl->AddControl(3,PwReLabel);
	TabControl->AddControl(3,PwOldEdit);
	TabControl->AddControl(3,PwNewEdit);
	TabControl->AddControl(3,PwReEdit);
	TabControl->AddControl(3,PwApplyBtn);
	PwApplyBtn->OnClick((TWndOnClick)PwApplyBtnClick);

	// misc tab page
	TabControl->AddControl(4,MiKeysBevel);
	TabControl->AddControl(4,MiKeysLabel);
	TabControl->AddControl(4,MiCycleLabel);
	TabControl->AddControl(4,MiCycleEdit);
	TabControl->AddControl(4,MiRestartLabel);
	TabControl->AddControl(4,MiRestartEdit);
	TabControl->AddControl(4,MiRebootLabel);
	TabControl->AddControl(4,MiRebootEdit);
	TabControl->AddControl(4,MiShutdownLabel);
	TabControl->AddControl(4,MiShutdownEdit);
	TabControl->AddControl(4,MiPowerOff);

	TabControl->AddControl(4,MiHandlingBevel);
	TabControl->AddControl(4,MiHandlingLabel);
	TabControl->AddControl(4,MiDisplayIndex);
	TabControl->AddControl(4,MiBootSelect);
	TabControl->AddControl(4,MiActiveAllow);
	TabControl->AddControl(4,MiAutoReveal);

	TabControl->AddControl(4,MiInstBootBevel);
	TabControl->AddControl(4,MiInstBootLabel);
	TabControl->AddControl(4,MiIgnoreClick);
	TabControl->AddControl(4,MiSetInstBoot);


}

void CPreference::InstallControls()
{
	// main
	Form->AddControl(TabBevel);
	Form->AddControl(TabControl);
	Form->AddControl(ResetBtn);
	Form->AddControl(SaveBtn);
	Form->AddControl(CloseBtn);
	Screen->AddWindow(Form);
	// graphics
	Form->AddControl(GrModeBevel);
	Form->AddControl(GrVideoMode);
	Form->AddControl(GrModeSlider);
	Form->AddControl(GrModeValue);
	Form->AddControl(GrTest);
	Form->AddControl(GrApply);

	Form->AddControl(GrPersonalBevel);
	Form->AddControl(GrPersonalLabel);
	Form->AddControl(GrFadeIn);
	Form->AddControl(GrFadeOut);

	Form->AddControl(GrVisualBevel);
	Form->AddControl(GrVisualLabel);
	Form->AddControl(GrSWCWD);
	Form->AddControl(GrFont);
	Form->AddControl(GrInvCursor);
	Form->AddControl(GrWallpaper);

	Form->AddControl(GrEnhanceBevel);
	Form->AddControl(GrEnhanceLabel);
	Form->AddControl(GrULFB);


	// color
	Form->AddControl(ClSchemeGroup);
	Form->AddControl(ClColorScheme);
	Form->AddControl(ClListBox);
	Form->AddControl(ClScrollBar);
	Form->AddControl(ClAdjustGroup);
	Form->AddControl(ClAdjustment);
	Form->AddControl(ClHueLabel);
	Form->AddControl(ClSatLabel);
	Form->AddControl(ClLumLabel);
	Form->AddControl(ClHueValue);
	Form->AddControl(ClSatValue);
	Form->AddControl(ClLumValue);
	Form->AddControl(ClHueSlider);
	Form->AddControl(ClSatSlider);
	Form->AddControl(ClLumSlider);
	Form->AddControl(ClFadeGroup);
	Form->AddControl(ClFadeColor);
	Form->AddControl(ClFadeIntSlider);
	Form->AddControl(ClFadeIntLabel);
	Form->AddControl(ClFadeIntValue);
	Form->AddControl(ClFadeTestBtn);
	// mouse
	Form->AddControl(MoTypeGroup);
	Form->AddControl(MoTypeLabel);
	Form->AddControl(MoMouseGroup);
	Form->AddControl(MoApply);
	Form->AddControl(MoSpeedGroup);
	Form->AddControl(MoSpeedLabel);
	Form->AddControl(MoSpeedSlider);
	Form->AddControl(MoSamplingGroup);
	Form->AddControl(MoSamplingLabel);
	Form->AddControl(MoSamplingSlider);
	// password
	Form->AddControl(PwGroup);
	Form->AddControl(PwLabel);
	Form->AddControl(PwOldLabel);
	Form->AddControl(PwNewLabel);
	Form->AddControl(PwReLabel);
	Form->AddControl(PwOldEdit);
	Form->AddControl(PwNewEdit);
	Form->AddControl(PwReEdit);
	Form->AddControl(PwApplyBtn);

	// misc tab page
	Form->AddControl(MiKeysBevel);
	Form->AddControl(MiKeysLabel);
	Form->AddControl(MiCycleLabel);
	Form->AddControl(MiCycleEdit);
	Form->AddControl(MiRestartLabel);
	Form->AddControl(MiRestartEdit);
	Form->AddControl(MiRebootLabel);
	Form->AddControl(MiRebootEdit);
	Form->AddControl(MiShutdownLabel);
	Form->AddControl(MiShutdownEdit);
	Form->AddControl(MiPowerOff);

	Form->AddControl(MiHandlingBevel);
	Form->AddControl(MiHandlingLabel);
	Form->AddControl(MiDisplayIndex);
	Form->AddControl(MiBootSelect);
	Form->AddControl(MiActiveAllow);
	Form->AddControl(MiAutoReveal);

	Form->AddControl(MiInstBootBevel);
	Form->AddControl(MiInstBootLabel);
	Form->AddControl(MiIgnoreClick);
	Form->AddControl(MiSetInstBoot);
}

void CPreference::InitializeData()
{
	InitGraphData();
	InitColorData();
	InitMouseData();
	InitMiscPref();
}

void CPreference::InitGraphData()
{
	TGraphData *GraphData;

	GraphData = XOSLData->GetGraphics();

	GrModeSlider->SetValue(Graph->GetModeIndex((TGraphMode)GraphData->VideoMode));
	if (!Graph->LinearSupport())
		GrULFB->Disable();
	else
		GrULFB->SetChecked(GraphData->LinearFB);
	GrSWCWD->SetChecked(!GraphData->FrameMove);
	GrFont->SetChecked(GraphData->Font9);
	GrInvCursor->SetChecked(GraphData->CursorInverted);
	GrWallpaper->SetChecked(GraphData->UseWallpaper);
	GrFadeOut->SetChecked(GraphData->DisableFadeOut);
	GrFadeIn->SetChecked(GraphData->DisableFadeIn);
	// handlers
	GrULFB->OnChange((TCheckBoxChange)GrULFBChange);
	GrTest->OnClick((TWndOnClick)GrModeTest);
	GrApply->OnClick((TWndOnClick)GrChangeMode);
	GrFadeOut->OnChange((TCheckBoxChange)GrFadeOutChange);
	GrFadeIn->OnChange((TCheckBoxChange)GrFadeInChange);

}

void CPreference::InitColorData()
{
	TColorData *ColorData;

	ColorData = XOSLData->GetColor();

	ClListBox->SetItemIndex(ColorData->ColorScheme);
//	ClListBox->SetItemIndex(ColorData->Reserved[0]);
	ClHueSlider->SetValue(ColorData->Hue);
	ClSatSlider->SetValue(ColorData->Saturation);
	ClLumSlider->SetValue(ColorData->Luminance);
	ClFadeIntSlider->SetValue(ColorData->FadeColorInt);
}

void CPreference::InitMouseData()
{
	TMouseData *MouseData;

	MouseData = XOSLData->GetMouse();

	MoMouseGroup->SetChecked(MouseData->MouseType);
	MoSpeedSlider->SetValue(MouseData->MouseSpeed);
	MoSamplingSlider->SetValue(MouseData->PS2Sampling);
	MoSpeedChange(this,MouseData->MouseSpeed);
	MoSamplingChange(this,MouseData->PS2Sampling);
	// handlers
	MoSpeedSlider->OnChange((TTrackBarChange)MoSpeedChange);
	MoSamplingSlider->OnChange((TTrackBarChange)MoSamplingChange);
	MoApply->OnClick((TWndOnClick)MoApplyClick);
}

void CPreference::InitMiscPref()
{
	TMiscPref *MiscPref;

	MiscPref = XOSLData->GetMiscPref();
	MiDisplayIndex->SetChecked(MiscPref->DisplayIndex);
	MiBootSelect->SetChecked(MiscPref->BootOnSelect);
	MiActiveAllow->SetChecked(MiscPref->ActiveAllow);
	MiAutoReveal->SetChecked(MiscPref->AutoReveal);
	MiIgnoreClick->SetChecked(MiscPref->IgnoreMouseClick);
	MiSetInstBoot->SetChecked(MiscPref->SetInstBoot);


	CreateKeyStr(MiRebootEdit,MiscPref->KeyColdReboot);
	CreateKeyStr(MiShutdownEdit,MiscPref->KeyShutdown);
	CreateKeyStr(MiRestartEdit,MiscPref->KeyRestart);
	CreateKeyStr(MiCycleEdit,MiscPref->KeyCycle);
	MiPowerOff->SetChecked(MiscPref->SoftPowerOff);

	// handlers for Misc tab
	MiDisplayIndex->OnChange((TCheckBoxChange)MiDisplayIndexChange);
	MiBootSelect->OnChange((TCheckBoxChange)MiBootSelectChange);
	MiActiveAllow->OnChange((TCheckBoxChange)MiActiveAllowChange);
	MiAutoReveal->OnChange((TCheckBoxChange)MiAutoRevealChange);
	MiIgnoreClick->OnChange((TCheckBoxChange)MiIgnoreClickChange);
	MiSetInstBoot->OnChange((TCheckBoxChange)MiSetInstBootChange);

	MiCycleEdit->OnKeyPress((TWndOnKeyPress)MiCycleEditKeyPress);
	MiRebootEdit->OnKeyPress((TWndOnKeyPress)MiRebootEditKeyPress);
	MiShutdownEdit->OnKeyPress((TWndOnKeyPress)MiShutdownEditKeyPress);
	MiRestartEdit->OnKeyPress((TWndOnKeyPress)MiRestartEditKeyPress);
	MiPowerOff->OnChange((TCheckBoxChange)MiPowerOffChange);
}

void CPreference::CenterWindow()
{
	int Width, Height;

	Graph->GetModeMetrics(Width,Height);
	Form->SetPosition((Width - 478) >> 1,(Height - 385) >> 1);
}

void CPreference::RealignText()
{
	int Visible;

	Visible = GrVideoMode->IsVisible();
	GrVideoMode->SetVisible(FALSE);
	GrVideoMode->SetCaption(StrGrScrnArea);
	GrVideoMode->SetVisible(Visible);

	Visible = GrPersonalLabel->IsVisible();
	GrPersonalLabel->SetVisible(FALSE);
	GrPersonalLabel->SetCaption(StrGrPersonal);
	GrPersonalLabel->SetVisible(Visible);

	Visible = GrVisualLabel->IsVisible();
	GrVisualLabel->SetVisible(FALSE);
	GrVisualLabel->SetCaption(StrGrVisual);
	GrVisualLabel->SetVisible(Visible);

	Visible = GrEnhanceLabel->IsVisible();
	GrEnhanceLabel->SetVisible(FALSE);
	GrEnhanceLabel->SetCaption(StrGrEnhance);
	GrEnhanceLabel->SetVisible(Visible);

	GrModeSliderChange(this,GrModeSlider->GetValue());

	Visible = ClColorScheme->IsVisible();
	ClColorScheme->SetVisible(FALSE);
	ClColorScheme->SetCaption(StrClColorScheme);
	ClColorScheme->SetVisible(Visible);

	Visible = ClAdjustment->IsVisible();
	ClAdjustment->SetVisible(FALSE);
	ClAdjustment->SetCaption(StrClAdjustment);
	ClAdjustment->SetVisible(Visible);

	Visible = ClFadeColor->IsVisible();
	ClFadeColor->SetVisible(FALSE);
	ClFadeColor->SetCaption(StrClFadeColor);
	ClFadeColor->SetVisible(Visible);

	Visible = MoTypeLabel->IsVisible();
	MoTypeLabel->SetVisible(FALSE);
	MoTypeLabel->SetCaption(StrMoTypeLabel);
	MoTypeLabel->SetVisible(Visible);

	Visible = MoSpeedLabel->IsVisible();
	MoSpeedLabel->SetVisible(FALSE);
	MoSpeedLabel->SetCaption(StrMoSpeedLabel);
	MoSpeedLabel->SetVisible(Visible);

	Visible = MoSamplingLabel->IsVisible();
	MoSamplingLabel->SetVisible(FALSE);
	MoSamplingLabel->SetCaption(StrMoSamplingLabel);
	MoSamplingLabel->SetVisible(Visible);

	Visible = PwLabel->IsVisible();
	PwLabel->SetVisible(FALSE);
	PwLabel->SetCaption(StrPwLabel);
	PwLabel->SetVisible(Visible);

	Visible = MiKeysLabel->IsVisible();
	MiKeysLabel->SetVisible(FALSE);
	MiKeysLabel->SetCaption(StrMiscKeys);
	MiKeysLabel->SetVisible(Visible);

	Visible = MiHandlingLabel->IsVisible();
	MiHandlingLabel->SetVisible(FALSE);
	MiHandlingLabel->SetCaption(StrHandling);
	MiHandlingLabel->SetVisible(Visible);

	Visible = MiInstBootLabel->IsVisible();
	MiInstBootLabel->SetVisible(FALSE);
	MiInstBootLabel->SetCaption(StrMiInstBoot);
	MiInstBootLabel->SetVisible(Visible);
}

void CPreference::CreateKeyStr(CEdit *Edit, int Key)
{
	char KeyName[64];

	Keyboard->GetKeyName(Key,KeyName);
	Edit->SetText(KeyName);

}


// graphics tab control handlers
void CPreference::GrModeSliderChange(CPreference *Preference, int Value)
{
	const char *ModeName;
	TGraphMode Mode;

	Mode = Graph->GetModeFromIndex(Value);
	ModeName = Graph->GetModeName(Mode);
	Preference->GrModeValue->SetCaption(ModeName);

}

void CPreference::GrULFBChange(CPreference *Preference, int Checked)
{
	Graph->VBE2(Checked);
	Preference->XOSLData->GetGraphics()->LinearFB = Checked;
}

void CPreference::GrSWCWDChange(CPreference *Preference, int Checked)
{
	Preference->Form->FrameMove = !Checked;
	Preference->XOSLData->GetGraphics()->FrameMove = Preference->Form->FrameMove;
}

void CPreference::CloseBtnClick(CPreference *Preference)
{
	Preference->Form->Hide();
	Preference->Loader->Focus();
}

void CPreference::ColorSchemeSelect(CPreference *Preference, int ItemIndex)
{
	Graph->Palette->SetScheme(ItemIndex);
	Graph->Palette->Install();
	Preference->XOSLData->GetColor()->ColorScheme = ItemIndex;
}

void CPreference::ChangeHue(CPreference *Preference, int Hue)
{
	CString HueMsg;

	Graph->Palette->SetHue(Hue);
	Graph->Palette->Install();
	HueMsg = (long) ( (360 * Hue) / 63 );
	HueMsg += "";
	Preference->ClHueValue->SetCaption(HueMsg);
	Preference->XOSLData->GetColor()->Hue = Hue;
}

void CPreference::ChangeSat(CPreference *Preference, int Sat)
{
	CString SatMsg;

	Graph->Palette->SetSaturation(Sat);
	Graph->Palette->Install();
	SatMsg = (long) ( (100 * Sat) / 63 );
	SatMsg += "%";
	Preference->ClSatValue->SetCaption(SatMsg);
	Preference->XOSLData->GetColor()->Saturation = Sat;
}

void CPreference::ChangeLum(CPreference *Preference, int Lum)
{
	CString LumMsg;

	if (Lum < 0)
		Graph->Palette->SetColorLayer(0,0,0,-Lum);
	else
		Graph->Palette->SetColorLayer(63,63,63,Lum);
	Graph->Palette->Install();
	LumMsg = (long) ( (100 * Lum) / 64 );
	LumMsg += "%";
	Preference->ClLumValue->SetCaption(LumMsg);
	Preference->XOSLData->GetColor()->Luminance = Lum;
}

void CPreference::TabChange(CPreference *Preference, int ActiveTab)
{
	if (ActiveTab == 3)
		Preference->ResetBtn->Disable();
	else
		Preference->ResetBtn->Enable();
}

void CPreference::GrFontChange(CPreference *Preference, int Checked)
{
	if (Checked)
		Graph->SetFont(AdditionalFont);
	else
		Graph->SetFont(DefaultFont);
	Preference->Setup->RealignText();
	Preference->RealignText();
	Preference->Loader->RealignText();
	Preference->About->RealignText();

	Preference->Screen->Refresh();

	Preference->XOSLData->GetGraphics()->Font9 = Checked;
}

void CPreference::ResetBtnClick(CPreference *Preference)
{
	switch (Preference->TabControl->GetActiveTab()) {
		case TAB_GRAPHICS:
			Preference->XOSLData->ResetGraphics();
			Preference->InitGraphData();
			break;
		case TAB_COLOR:
			Preference->XOSLData->ResetColor();
			Preference->InitColorData();
			break;
		case TAB_MOUSE:
			Preference->XOSLData->ResetMouse();
			Preference->InitMouseData();
			break;
		case TAB_MISC:
			Preference->XOSLData->ResetMiscPref();
			Preference->InitMiscPref();
			break;
		default:
			break;
	}
}

void CPreference::SaveBtnClick(CPreference *Preference)
{
	Preference->XOSLData->SaveData();
	Preference->Dialogs->ShowMessageDialog("Save","Data saved.");
}

void CPreference::GrInvCursorChange(CPreference *Preference, int Checked)
{
	int X, Y;

	InvertCursor(Checked);
	Graph->GetCursorXY(X,Y);
	Graph->SetCursorXY(X,Y);
	Preference->XOSLData->GetGraphics()->CursorInverted = Checked;
}

void CPreference::GrModeTest(CPreference *Preference)
{
	int Status;
	TGraphMode LastMode;
	TGraphMode NewMode;

	LastMode = Graph->GetMode();
	NewMode = Preference->GetSelectedVideoMode();

	Graph->HideCursor();
	Status = Preference->Screen->TestGraphicsMode(NewMode);
	if (Status != -1) {
		while (!Preference->Keyboard->KbHit() && !Preference->Mouse->MouseDown());
		if (Preference->Keyboard->KbHit())
			Preference->Keyboard->GetCh();
	}


	Graph->Palette->SetSaturation(Preference->XOSLData->GetColor()->Saturation);
	Preference->Screen->SetGraphicsMode(LastMode);
	Graph->ShowCursor();
}

void CPreference::GrChangeMode(CPreference *Preference)
{
	TGraphMode LastMode;
	TGraphMode NewMode;
	int Width, Height;

	LastMode = Graph->GetMode();
	NewMode = Preference->GetSelectedVideoMode();
	if (Preference->Screen->SetGraphicsMode(NewMode) == -1)
		Preference->Screen->SetGraphicsMode(LastMode);
	else {
		Graph->GetModeMetrics(Width,Height);
		Preference->Mouse->SetWindow(0,0,Width,Height);
		Preference->XOSLData->GetGraphics()->VideoMode = NewMode;
		/* Relocate (center) all windows */
		Preference->Loader->CenterWindow();
		Preference->Setup->CenterWindow();
		Preference->CenterWindow();
		Preference->About->CenterWindow();
	}
}

TGraphMode CPreference::GetSelectedVideoMode()
{
	TGraphMode VideoMode;

	VideoMode = Graph->GetModeFromIndex(GrModeSlider->GetValue());
	switch (VideoMode) {
		case 1:
			return mode640x480;
		case 2:
			return mode800x600;
		case 3:
			return mode1024x768;
		case 4:
			return mode1280x1024;
		case 5:
			return mode1600x1200;
		default:
			return mode640x480;
	}
}

void CPreference::MoSpeedChange(CPreference *Preference, int Value)
{
	TMouseSpeed MouseSpeed;

	Preference->XOSLData->GetMouse()->MouseSpeed = Value;
	if (Preference->Mouse->GetMouse() == MOUSE_PS2) {
		Preference->Mouse->GetSpeed(MouseSpeed);
		Value >>= 1;
		if (Value == 4) {
			MouseSpeed.PS2.Scale = 2;
			MouseSpeed.PS2.Resolution = 3;
		}
		else {
			MouseSpeed.PS2.Scale = 1;
			MouseSpeed.PS2.Resolution = Value;
		}
	}
	else
		MouseSpeed.Com.Speed = Value;
	Preference->Mouse->SetSpeed(MouseSpeed);
}

void CPreference::MoSamplingChange(CPreference *Preference, int Value)
{
	TMouseSpeed MouseSpeed;

	Preference->XOSLData->GetMouse()->PS2Sampling = Value;
	if (Preference->Mouse->GetMouse() == MOUSE_PS2) {
		Preference->Mouse->GetSpeed(MouseSpeed);
		MouseSpeed.PS2.Sampling = Value;
		Preference->Mouse->SetSpeed(MouseSpeed);
	}
}

void CPreference::MoApplyClick(CPreference *Preference)
{
	int LastPort;
	int MousePort;
	int Checked;
	CString Message;
	static const char *MsgBoxTitle = "Mouse";
	static const int PortList[] = {
		MOUSE_COM1,MOUSE_COM2,MOUSE_COM3,MOUSE_COM4,MOUSE_PS2
	};

	Graph->HideCursor();
	LastPort = Preference->Mouse->GetMouse();
	Checked = Preference->MoMouseGroup->GetChecked();
	if (Checked == 5) {
		Preference->Mouse->SetMouse(-1);
		Preference->XOSLData->GetMouse()->MouseType = -1;
	}
	else {
		MousePort = PortList[Checked];
		Message = "Initializing ";
		Message += Preference->Mouse->MousePortName(MousePort);
		Message += "...";
		Preference->Dialogs->ShowMessageDialog(MsgBoxTitle,Message);
		if (Preference->Mouse->SetMouse(MousePort) == -1) {
			Preference->Mouse->SetMouse(LastPort);
			Message += "failed!";
			Preference->Dialogs->ShowMessageDialog(MsgBoxTitle,Message);
			if (LastPort != -1)
				Graph->ShowCursor();
		}
		else {
			Preference->XOSLData->GetMouse()->MouseType = MousePort;
			Message += "done";
			Preference->Dialogs->ShowMessageDialog(MsgBoxTitle,Message);
			Graph->ShowCursor();
		}
		MoSpeedChange(Preference,Preference->MoSpeedSlider->GetValue());
		MoSamplingChange(Preference,Preference->MoSamplingSlider->GetValue());
	}
}

void CPreference::PwApplyBtnClick(CPreference *Preference)
{
	unsigned long Code;
	unsigned long OldCode;
	char Password[48];

	OldCode = Preference->XOSLData->GetPassword();
	Preference->PwOldEdit->GetText(Password);
	Code = EncodePassword(Password);
	if (Code != OldCode) {
		Preference->Dialogs->ShowMessageDialog("Old password","Invalid password.");
		return;
	}
	Preference->PwNewEdit->GetText(Password);
	Code = EncodePassword(Password);
	Preference->PwReEdit->GetText(Password);
	if (Code != EncodePassword(Password)) {
		Preference->Dialogs->ShowMessageDialog("New password","Passwords do not match.");
		return;
	}
	Preference->XOSLData->SetPassword(Code);
	Preference->XOSLData->SaveData();
	Preference->Dialogs->ShowMessageDialog("Password","New password saved.");
}

void CPreference::GrWallpaperChange(CPreference *Preference, int Checked)
{
	if (!NoUpdates) {
		Preference->Screen->SetUseWallpaper(Checked);
		Preference->Screen->Refresh();
		Preference->XOSLData->GetGraphics()->UseWallpaper = Checked;
	}
}

void CPreference::GrFadeOutChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetGraphics()->DisableFadeOut = Checked;
}

void CPreference::GrFadeInChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetGraphics()->DisableFadeIn = Checked;
}

void CPreference::FadeIntChange(CPreference *Preference, int Intensity)
{
	CString IntValue;

	Preference->XOSLData->GetColor()->FadeColorInt = Intensity;
	IntValue = Intensity;
	IntValue += "%";
	Preference->ClFadeIntValue->SetCaption(IntValue);
}

void CPreference::FadeTestClick(CPreference *Preference)
{
	unsigned long TimerTicks;

	Graph->Palette->LockFading();
	Graph->Palette->SetFadeOutColor(Preference->XOSLData->GetColor()->FadeColorInt);
	Graph->Palette->FadeOut();
	for (TimerTicks = GetTimerTicks() + 18; TimerTicks != GetTimerTicks(););
	Graph->Palette->FadeIn();
}


void CPreference::MiDisplayIndexChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetMiscPref()->DisplayIndex = Checked;
	Preference->Loader->CreateBootList(Preference->Loader->GetBootItemIndex());
}

void CPreference::MiBootSelectChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetMiscPref()->BootOnSelect = Checked;
}

void CPreference::MiActiveAllowChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetMiscPref()->ActiveAllow = Checked;
}

void CPreference::MiAutoRevealChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetMiscPref()->AutoReveal = Checked;
}


void CPreference::MiIgnoreClickChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetMiscPref()->IgnoreMouseClick = Checked;
}

void CPreference::MiSetInstBootChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetMiscPref()->SetInstBoot = Checked;
}

void CPreference::MiCycleEditKeyPress(CPreference *Preference, int &Key)
{
	char KeyName[64];

	Preference->XOSLData->GetMiscPref()->KeyCycle = Key;
	Preference->SetIgnoreNextKey();
	Preference->Keyboard->GetKeyName(KeyName);
	Preference->MiCycleEdit->SetText(KeyName);
	Key = KEY_END;
}

void CPreference::MiRebootEditKeyPress(CPreference *Preference, int &Key)
{
	char KeyName[64];

	Preference->XOSLData->GetMiscPref()->KeyColdReboot = Key;
	Preference->SetIgnoreNextKey();
	Preference->Keyboard->GetKeyName(KeyName);
	Preference->MiRebootEdit->SetText(KeyName);
	Key = KEY_END;
}

void CPreference::MiShutdownEditKeyPress(CPreference *Preference, int &Key)
{
	char KeyName[64];

	Preference->XOSLData->GetMiscPref()->KeyShutdown = Key;
	Preference->SetIgnoreNextKey();
	Preference->Keyboard->GetKeyName(KeyName);
	Preference->MiShutdownEdit->SetText(KeyName);
	Key = KEY_END;
}

void CPreference::MiRestartEditKeyPress(CPreference *Preference, int &Key)
{
	char KeyName[64];

	Preference->XOSLData->GetMiscPref()->KeyRestart = Key;
	Preference->SetIgnoreNextKey();
	Preference->Keyboard->GetKeyName(KeyName);
	Preference->MiRestartEdit->SetText(KeyName);
	Key = KEY_END;
}

void CPreference::MiPowerOffChange(CPreference *Preference, int Checked)
{
	Preference->XOSLData->GetMiscPref()->SoftPowerOff = Checked;
}
