/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#ifndef __settings__
#define __settings__

#include <gui.h>
#include <setup.h>
#include <loader.h>
#include <about.h>

#include <screen.h>
#include <graph.h>
#include <cstring.h>
#include <xosldata.h>
#include <dcx.h>
#include <key.h>
#include <mouse.h>
#include <dialogs.h>
#include <encpwd.h>
#include <timer.h>


class CPreference {
	public:
		CPreference(CLoader *LoaderToUse, CSetup *SetupToUse,
			CAbout *AboutToUse, CScreen *ScreenToUse, CXOSLData *XOSLDataToUse,
			CKeyboard *KeyboardToUse, CMouse *MouseToUse);
		~CPreference();

		void Show();
		void Hide();
		int IsVisible();
		int GotFocus();

		void CenterWindow();
		void RealignText();
		int GetIgnoreNextKey();
		void InitializeData();
	private:
		CLoader *Loader;
		CSetup *Setup;
		CAbout *About;
		CScreen *Screen;
		CXOSLData *XOSLData;
		CKeyboard *Keyboard;
		CMouse *Mouse;

		CDialogs *Dialogs;

	private:
		static int NoUpdates;

		static void CloseBtnClick(CPreference *Preference);
		static void TabChange(CPreference *Preference, int ActiveTab);
		static void ResetBtnClick(CPreference *Preference);
		static void SaveBtnClick(CPreference *Preference);

		static void GrModeSliderChange(CPreference *Preference, int Value);
		static void GrULFBChange(CPreference *Preference, int Checked);
		static void GrSWCWDChange(CPreference *Preference, int Checked);
		static void GrFontChange(CPreference *Preference, int Checked);
		static void GrInvCursorChange(CPreference *Preference, int Checked);
		static void GrModeTest(CPreference *Preference);
		static void GrChangeMode(CPreference *Preference);

		TGraphMode GetSelectedVideoMode();


		static void GrWallpaperChange(CPreference *Preference, int Checked);
		static void GrFadeOutChange(CPreference *Preference, int Checked);
		static void GrFadeInChange(CPreference *Preference, int Checked);

		static void ColorSchemeSelect(CPreference *Preference, int ItemIndex);
		static void ChangeHue(CPreference *Preference, int Hue);
		static void ChangeSat(CPreference *Preference, int Sat);
		static void ChangeLum(CPreference *Preference, int Lum);
		static void FadeIntChange(CPreference *Preference, int Intensity);
		static void FadeTestClick(CPreference *Preference);

		static void MoSpeedChange(CPreference *Preference, int Value);
		static void MoSamplingChange(CPreference *Preference, int Value);
		static void MoApplyClick(CPreference *Preference);
	
		static void PwApplyBtnClick(CPreference *Preference);

		static void MiCycleEditKeyPress(CPreference *Preference, int &Key);
		static void MiRebootEditKeyPress(CPreference *Preference, int &Key);
		static void MiShutdownEditKeyPress(CPreference *Preference, int &Key);
		static void MiRestartEditKeyPress(CPreference *Preference, int &Key);
		static void MiPowerOffChange(CPreference *Preference, int Checked);
		static void MiDisplayIndexChange(CPreference *Preference, int Checked);
		static void MiBootSelectChange(CPreference *Preference, int Checked);
		static void MiActiveAllowChange(CPreference *Preference, int Checked);
		static void MiAutoRevealChange(CPreference *Preference, int Checked);

		static void MiIgnoreClickChange(CPreference *Preference, int Checked);
		static void MiSetInstBootChange(CPreference *Preference, int Checked);


	private:
		void SetIgnoreNextKey();

	private:



		// main
		CForm *Form;
		CBevel *TabBevel;
		CTabControl *TabControl;
		CButton *ResetBtn;
		CButton *SaveBtn;
		CButton *CloseBtn;
		// graphics
		CBevel *GrModeBevel;
		CLabel *GrVideoMode;
		CTrackBar *GrModeSlider;
		CLabel *GrModeValue;
		CButton *GrTest;
		CButton *GrApply;

		CBevel *GrPersonalBevel;
		CLabel *GrPersonalLabel;
		CCheckBox *GrFadeIn;
		CCheckBox *GrFadeOut;

		CBevel *GrVisualBevel;
		CLabel *GrVisualLabel;
		CCheckBox *GrSWCWD;
		CCheckBox *GrFont;
		CCheckBox *GrInvCursor;
		CCheckBox *GrWallpaper;

		CBevel *GrEnhanceBevel;
		CLabel *GrEnhanceLabel;
		CCheckBox *GrULFB;

		// color
		CBevel *ClSchemeGroup;
		CLabel *ClColorScheme;
		CListBox *ClListBox;
		CScrollBar *ClScrollBar;
		CBevel *ClAdjustGroup;
		CLabel *ClAdjustment;
		CLabel *ClHueLabel;
		CLabel *ClSatLabel;
		CLabel *ClLumLabel;
		CLabel *ClHueValue;
		CLabel *ClSatValue;
		CLabel *ClLumValue;
		CTrackBar *ClHueSlider;
		CTrackBar *ClSatSlider;
		CTrackBar *ClLumSlider;

		CBevel *ClFadeGroup;
		CLabel *ClFadeColor;
		CLabel *ClFadeIntLabel;
		CTrackBar *ClFadeIntSlider;
		CLabel *ClFadeIntValue;
		CButton *ClFadeTestBtn;

		// mouse
		CBevel *MoTypeGroup;
		CLabel *MoTypeLabel;
		CRadioGroup *MoMouseGroup;

		CBevel *MoSpeedGroup;
		CLabel *MoSpeedLabel;
		CTrackBar *MoSpeedSlider;
		CBevel *MoSamplingGroup;
		CLabel *MoSamplingLabel;
		CTrackBar *MoSamplingSlider;
		CButton *MoApply;
		// password
		CBevel *PwGroup;
		CLabel *PwLabel;
		CLabel *PwOldLabel;
		CLabel *PwNewLabel;
		CLabel *PwReLabel;
		CEdit *PwOldEdit;
		CEdit *PwNewEdit;
		CEdit *PwReEdit;
		CButton *PwApplyBtn;

		// misc
		CBevel *MiKeysBevel;
		CLabel *MiKeysLabel;
		CLabel *MiCycleLabel;
		CEdit *MiCycleEdit;
		CLabel *MiRestartLabel;
		CEdit *MiRestartEdit;
		CLabel *MiRebootLabel;
		CEdit *MiRebootEdit;
		CLabel *MiShutdownLabel;
		CEdit *MiShutdownEdit;
		CCheckBox *MiPowerOff;

		CBevel *MiHandlingBevel;
		CLabel *MiHandlingLabel;
		CCheckBox *MiDisplayIndex;
		CCheckBox *MiBootSelect;
		CCheckBox *MiActiveAllow;
		CCheckBox *MiAutoReveal;

		CBevel *MiInstBootBevel;
		CLabel *MiInstBootLabel;
		CCheckBox *MiIgnoreClick;
		CCheckBox *MiSetInstBoot;



		void InitGraphData();
		void InitColorData();
		void InitMouseData();
		void InitMiscPref();
	private:
		void CreateControls();
		void InitializeControls();
		void InstallControls();
      void CreateKeyStr(CEdit *Edit, int Key);

		int IgnoreKey;

};



#endif