/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#include <setup.h>

////


static const char *StrYes = "yes";
static const char *StrNo = "no";

static const char *StrMiTimerLabel = "Instant Boot";
static const char *StrMiKeyLabel = "Additional keys";
static const char *StrMiHotkeyLabel = "Hotkey";
static const char *StrMiPwdLabel = "Password";

//////////


int CSetup::UpdateIB = 1;
int CSetup::HidingUpdate;


CSetup::CSetup(CLoader *LoaderToUse, CScreen *ScreenToUse, 
				CPartList *PartListToUse, CBootItems *BootItemsToUse,
				CKeyboard *KeyboardToUse)
{
	Loader = LoaderToUse;
	Screen = ScreenToUse;
	PartList = PartListToUse;
	BootItems = BootItemsToUse;
	Keyboard = KeyboardToUse;

	PartitionDrawn = FALSE;
	CreateControls();
	InitializeControls();
	InstallControls();
	Dialogs = new CDialogs(ScreenToUse,Form,this);
}

CSetup::~CSetup()
{
}

void CSetup::Show()
{
	if (!Form->Visible) {
		Form->Show();
		Form->FocusControl(CloseBtn);
	}
	else
		Screen->SetFocusWnd(Form);
}


void CSetup::Hide()
{
	if (Form->Visible)
		Form->Hide();
}

int CSetup::IsVisible()
{
	return Form->Visible && Form->IsEnabled();
}

int CSetup::GotFocus()
{
	return Screen->GetFocusWnd() == Form;
}

void CSetup::CreateControls()
{
	// main controls
	Form = new CForm("XOSL Boot items configuration",FORM_NORMAL,TRUE,300,100,527,553,FALSE);
	ItemsList = new CListBox(4,TRUE,8,8,425,209,TRUE,this);
	ItemsScroll = new CScrollBar(0,0,0,FALSE,414,9,207,TRUE,ItemsList);
	RemoveBtn = new CButton("Remove",440,88,75,25,TRUE,this);
	MoveUpBtn = new CButton("Up",440,120,75,25,TRUE,this);
	MoveDownBtn = new CButton("Down",440,152,75,25,TRUE,this);
	CloneBtn = new CButton("Clone",440,192,75,25,TRUE,this);
	TabBevel = new CBevel(BEVEL_BOX,FALSE,8,224,425,274,TRUE);
	TabControl = new CTabControl(3,8,497,425,TRUE,this);
	ResetBtn = new CButton("Reset",440,400,75,25,TRUE,this);
	SaveBtn = new CButton("Save",440,432,75,25,TRUE,this);
	CloseBtn = new CButton("Close",440,472,75,25,TRUE,this);
	// partition
	PtListBox = new CListBox(4,TRUE,46,240,340,185,TRUE,this);
	PtScrollBar = new CScrollBar(0,0,0,FALSE,367,241,183,TRUE,PtListBox);
	PtAddBtn = new CButton("Add",224,444,75,25,TRUE,this);
	PtUpdateBtn = new CButton("Update",311,444,75,25,TRUE,this);
	PtLabel = new CLabel("Boot item name:",STYLE_REGULAR,TRUE,17,46,432,TRUE);
	PtItemName = new CEdit("",47,FALSE,FALSE,46,448,166,TRUE,this);

	// hiding
	HdListBox = new CListBox(4,TRUE,46,240,340,121,FALSE,this);
	HdScrollBar = new CScrollBar(0,0,0,FALSE,367,241,119,FALSE,HdListBox);
	HdHideBevel = new CBevel(BEVEL_FRAME,TRUE,48,376,97,97,FALSE);
	HdRadioGroup = new CRadioGroup(64,392,80,72,FALSE,this);
	HdIgnoreAll = new CButton("Ignore all",232,380,89,25,FALSE,this);
	HdHideAll = new CButton("Hide all",232,412,89,25,FALSE,this);
	HdRevealAll = new CButton("Reveal all",232,444,89,25,FALSE,this);
	// misc
	MiTimerBevel = new CBevel(BEVEL_FRAME,TRUE,24,240,209,72,FALSE);
	MiTimerLabel = new CLabel(StrMiTimerLabel,STYLE_REGULAR,FALSE,17,32,233,FALSE);

	MiTimerSlider = new CTrackBar(0,81,0,40,262,177,FALSE,this);
	MiTimeout = new CLabel("0 sec",STYLE_REGULAR,TRUE,17,160,284,FALSE);
	MiInstBoot = new CCheckBox("Instant Boot",FALSE,48,283,FALSE,this);

	MiKeyBevel = new CBevel(BEVEL_FRAME,TRUE,24,328,209,62,FALSE);
	MiKeyLabel = new CLabel(StrMiKeyLabel,STYLE_REGULAR,FALSE,17,32,321,FALSE);
	MiKeyEdit = new CEdit("",64,FALSE,FALSE,40,353,102,FALSE,this);
	MiKeyApply = new CButton("Apply",150,351,67,25,FALSE,this);

	MiHotkeyBevel = new CBevel(BEVEL_FRAME,TRUE,24,406,209,62,FALSE);
	MiHotkeyLabel = new CLabel(StrMiHotkeyLabel,STYLE_REGULAR,FALSE,17,32,399,FALSE);
	MiHotkeyEdit = new CEdit("",64,FALSE,FALSE,40,431,102,FALSE,this);
	MiHotkeyClearBtn = new CButton("Clear",150,429,67,25,FALSE,this);

	MiPwdBevel = new CBevel(BEVEL_FRAME,TRUE,240,240,169,193,FALSE);
	MiPwdLabel = new CLabel(StrMiPwdLabel,STYLE_REGULAR,FALSE,17,248,233,FALSE);
	MiNewLabel = new CLabel("New password",STYLE_REGULAR,TRUE,17,264,264,FALSE);
	MiReLabel = new CLabel("Re-enter password",STYLE_REGULAR,TRUE,17,264,320,FALSE);
	MiNewEdit = new CEdit("",32,TRUE,FALSE,264,280,121,FALSE,this);
	MiReEdit = new CEdit("",32,TRUE,FALSE,264,336,121,FALSE,this);
	MiPwdApply = new CButton("Apply",287,392,75,25,FALSE,this);
	MiActivate = new CCheckBox("Activate",FALSE,256,441,FALSE,this);
	MiDriveNumFix = new CCheckBox("Fix FAT drive number",FALSE,256,465,FALSE,this);
}

void CSetup::InitializeControls()
{
	// main
	ItemsList->SetColumn(0,160,"Name");
	ItemsList->SetColumn(1,60,"Activate");
	ItemsList->SetColumn(2,70,"Protected");
	ItemsList->SetColumn(3,115,"Additional keys");
	ItemsList->SetScrollBar(ItemsScroll);
	ItemsList->OnSelect((TListBoxSelect)ItemsListSelect);

	TabControl->SetTabCaption(0,"Partition");
	TabControl->SetTabCaption(1,"Hiding");
	TabControl->SetTabCaption(2,"Misc");
	TabControl->SetTabPage(TabBevel);
	CloseBtn->OnClick((TWndOnClick)CloseBtnClick);
	MoveUpBtn->OnClick((TWndOnClick)MoveUpBtnClick);
	MoveDownBtn->OnClick((TWndOnClick)MoveDownBtnClick);
	RemoveBtn->OnClick((TWndOnClick)RemoveBtnClick);
	CloneBtn->OnClick((TWndOnClick)CloneBtnClick);
	ResetBtn->OnClick((TWndOnClick)ResetBtnClick);
	SaveBtn->OnClick((TWndOnClick)SaveBtnClick);

	// partition
	PtListBox->SetColumn(0,50,"Disk");
	PtListBox->SetColumn(1,65,"Type");
	PtListBox->SetColumn(2,135,"System");
	PtListBox->SetColumn(3,70,"Size (Mb)");
	PtListBox->SetScrollBar(PtScrollBar);
	TabControl->AddControl(0,PtListBox);
	TabControl->AddControl(0,PtScrollBar);
	TabControl->AddControl(0,PtItemName);
	TabControl->AddControl(0,PtAddBtn);
	TabControl->AddControl(0,PtUpdateBtn);
	TabControl->AddControl(0,PtLabel);

	PtAddBtn->OnClick((TWndOnClick)PtAddBtnClick);
	PtUpdateBtn->OnClick((TWndOnClick)PtUpdateBtnClick);
	// hiding
	HdListBox->SetColumn(0,50,"Disk");
	HdListBox->SetColumn(1,65,"Type");
	HdListBox->SetColumn(2,135,"System");
	HdListBox->SetColumn(3,70,"Status");
	HdListBox->SetScrollBar(HdScrollBar);
	TabControl->AddControl(1,HdListBox);
	TabControl->AddControl(1,HdScrollBar);
	TabControl->AddControl(1,HdHideBevel);
	TabControl->AddControl(1,HdRadioGroup);
	TabControl->AddControl(1,HdIgnoreAll);
	TabControl->AddControl(1,HdHideAll);
	TabControl->AddControl(1,HdRevealAll);
	HdListBox->OnSelect((TListBoxSelect)HdListBoxSelect);
	HdIgnoreAll->OnClick((TWndOnClick)HdIgnoreAllClick);
	HdHideAll->OnClick((TWndOnClick)HdHideAllClick);
	HdRevealAll->OnClick((TWndOnClick)HdRevealAllClick);
	HdRadioGroup->AddButton("Ignore");
	HdRadioGroup->AddButton("Hide");
	HdRadioGroup->AddButton("Reveal");
	HdRadioGroup->OnSelect((TRadioGroupSelect)HdHidingSelect);

	// misc
	TabControl->AddControl(2,MiTimerBevel);
	TabControl->AddControl(2,MiTimerLabel);
	TabControl->AddControl(2,MiTimerSlider);
	TabControl->AddControl(2,MiTimeout);
	TabControl->AddControl(2,MiInstBoot);
	TabControl->AddControl(2,MiKeyBevel);
	TabControl->AddControl(2,MiKeyLabel);
	TabControl->AddControl(2,MiKeyEdit);
	TabControl->AddControl(2,MiKeyApply);
	TabControl->AddControl(2,MiHotkeyBevel);
	TabControl->AddControl(2,MiHotkeyLabel);
	TabControl->AddControl(2,MiHotkeyEdit);
	TabControl->AddControl(2,MiHotkeyClearBtn);

	TabControl->AddControl(2,MiPwdBevel);
	TabControl->AddControl(2,MiPwdLabel);
	TabControl->AddControl(2,MiNewLabel);
	TabControl->AddControl(2,MiReLabel);
	TabControl->AddControl(2,MiNewEdit);
	TabControl->AddControl(2,MiReEdit);
	TabControl->AddControl(2,MiPwdApply);
	TabControl->AddControl(2,MiActivate);
	TabControl->AddControl(2,MiDriveNumFix);
	MiActivate->OnChange((TCheckBoxChange)MiActivateChange);
	MiDriveNumFix->OnChange((TCheckBoxChange)MiDriveNumFixChange);
	MiInstBoot->OnChange((TCheckBoxChange)MiInstBootChange);
	MiPwdApply->OnClick((TWndOnClick)MiPwdApplyClick);
	MiTimerSlider->OnChange((TTrackBarChange)MiTimerSliderChange);
	MiKeyApply->OnClick((TWndOnClick)MiKeyApplyClick);
	MiHotkeyEdit->OnKeyPress((TWndOnKeyPress)MiHotkeyKeyPress);
	MiHotkeyClearBtn->OnClick((TWndOnClick)MiHotkeyClearBtnClick);
}

void CSetup::InstallControls()
{
	// main 1
	Form->AddControl(ResetBtn);
	Form->AddControl(SaveBtn);
	Form->AddControl(CloseBtn);
	Form->AddControl(ItemsList);
	Form->AddControl(ItemsScroll);
	Form->AddControl(RemoveBtn);
	Form->AddControl(MoveUpBtn);
	Form->AddControl(MoveDownBtn);
	Form->AddControl(CloneBtn);
	Screen->AddWindow(Form);
	// partition
	Form->AddControl(PtListBox);
	Form->AddControl(PtScrollBar);
	Form->AddControl(PtItemName);
	Form->AddControl(PtAddBtn);
	Form->AddControl(PtUpdateBtn);
	Form->AddControl(PtLabel);
	// hiding
	Form->AddControl(HdListBox);
	Form->AddControl(HdScrollBar);
	Form->AddControl(HdHideBevel);
	Form->AddControl(HdRadioGroup);
	Form->AddControl(HdIgnoreAll);
	Form->AddControl(HdHideAll);
	Form->AddControl(HdRevealAll);
	// misc
	Form->AddControl(MiTimerBevel);
	Form->AddControl(MiTimerLabel);
	Form->AddControl(MiTimerSlider);
	Form->AddControl(MiTimeout);
	Form->AddControl(MiInstBoot);
	Form->AddControl(MiKeyBevel);
	Form->AddControl(MiKeyLabel);
	Form->AddControl(MiKeyEdit);
	Form->AddControl(MiKeyApply);
	Form->AddControl(MiHotkeyBevel);
	Form->AddControl(MiHotkeyLabel);
	Form->AddControl(MiHotkeyEdit);
	Form->AddControl(MiHotkeyClearBtn);
	Form->AddControl(MiPwdBevel);
	Form->AddControl(MiPwdLabel);
	Form->AddControl(MiNewLabel);
	Form->AddControl(MiReLabel);
	Form->AddControl(MiNewEdit);
	Form->AddControl(MiReEdit);
	Form->AddControl(MiPwdApply);
	Form->AddControl(MiActivate);
	Form->AddControl(MiDriveNumFix);
	// main 2
	Form->AddControl(TabBevel);
	Form->AddControl(TabControl);

}

void CSetup::InitializeData()
{
	BootItemsDrawn = FALSE;
	InitNoItems();
	InitPartitions();
	InitHiding();
	if (BootItems->GetCount())
		ItemsList->Enable();
	InitBootItems();
}

void CSetup::ResetData()
{
	BootItemsDrawn = FALSE;
	BootItems->Reset();
	ItemsList->Clear();
	InitBootItems();
	InitNoItems();
	if (BootItems->GetCount())
		ItemsList->Enable();
}

void CSetup::InitPartitions()
{
	int Count;
	const TPartition *Partition;
	int Row;

	Count = PartList->GetCount();
	for (Row = 0; Row < Count; ++Row) {
		PtListBox->AddRow();

		Partition = PartList->GetPartition(Row);

		AddDisk(PtListBox,Row,Partition->Drive);
		AddType(PtListBox,Row,Partition->Type);
		AddSystem(PtListBox,Row,Partition->FSName);
		AddSize(PtListBox,Row,Partition->SectorCount);

	}
	PtListBox->SetItemIndex(0);
}

void CSetup::InitHiding()
{
	int Count;
	const TPartition *Partition;
	int Row;

	Count = PartList->GetCount();
	for (Row = 0; Row < Count; ++Row) {
		HdListBox->AddRow();
		Partition = PartList->GetPartition(Row);
		AddDisk(HdListBox,Row,Partition->Drive);
		AddType(HdListBox,Row,Partition->Type);
		AddSystem(HdListBox,Row,Partition->FSName);
	}
}


void CSetup::AddDisk(CListBox *ListBox, int Row, int Drive)
{
	CString Msg;

	if (Drive & 0x80)
		Msg = "HD";
	else
		Msg = "FD";
	Msg += (long)(Drive & 0x007f);
	ListBox->AddItem(Row,0,Msg);
}

void CSetup::AddType(CListBox *ListBox, int Row, int Type)
{
	static const char *Types[] = {
		"primary","logical","mbr","floppy"
	};

	ListBox->AddItem(Row,1,Types[Type]);
}

void CSetup::AddSystem(CListBox *ListBox, int Row, const char *FSName)
{
	ListBox->AddItem(Row,2,FSName);
}

void CSetup::AddSize(CListBox *ListBox, int Row, long Sectors)
{
	CString Msg;

	Msg = Sectors >> 11;
	ListBox->AddItem(Row,3,Msg);
}

void CSetup::InitBootItems()
{
	int Index, Count;
	const TBootItem *Entry;
	CString ItemName;
	char UnformatStr[64];

	Count = BootItems->GetCount();
	if (!BootItemsDrawn) {
		BootItemsDrawn = TRUE;
		for (Index = 0; Index < Count; ++Index)
			ItemsList->AddRow();
	}
	for (Index = 0; Index < Count; ++Index) {
		Entry = BootItems->GetBootItem(Index);
		if (!Entry->Valid)
			ItemName = "[!]";
		else
			ItemName = "";
		if (BootItems->GetInstantBoot() == Index)
			ItemName += "[ib]";
		ItemName += Entry->ItemName;
		ItemsList->AddItem(Index,0,ItemName);

		ItemsList->AddItem(Index,1,Entry->Activate ? StrYes : StrNo);
		ItemsList->AddItem(Index,2,Entry->Password ? StrYes : StrNo);

		UnformatKeys(Entry->Keys,UnformatStr);
		ItemsList->AddItem(Index,3,UnformatStr);
	}
}

void CSetup::InitHideStatus()
{
	const char *HidingItems;
	int Index, Count;
	const char *Status[] = {
		"Ignore","Hide","Reveal"
	};

	HidingItems = BootItems->GetBootItem(ItemsList->GetItemIndex())->HidingItems;
	Count = PartList->GetCount();
	for (Index = 0; Index < Count; ++Index)
		HdListBox->AddItem(Index,3,Status[HidingItems[Index]]);
}

void CSetup::InitNoItems()
{
	RemoveBtn->Disable();
	CloneBtn->Disable();
	MoveUpBtn->Disable();
	MoveDownBtn->Disable();
	ItemsList->Disable();
	PtUpdateBtn->Disable();
	HdListBox->Disable();
	HdIgnoreAll->Disable();
	HdHideAll->Disable();
	HdRevealAll->Disable();
	HdRadioGroup->Disable();
	MiKeyEdit->Disable();
	MiKeyApply->Disable();
	MiHotkeyEdit->Disable();
	MiHotkeyClearBtn->Disable();
	MiPwdApply->Disable();
	MiActivate->Disable();
	MiDriveNumFix->Disable();
	MiActivate->SetChecked(FALSE);
	MiInstBoot->Disable();
	MiInstBoot->SetChecked(FALSE);
}

void CSetup::CenterWindow()
{
	int Width, Height;

	Graph->GetModeMetrics(Width,Height);
	Form->SetPosition((Width - 527) >> 1,(Height - 553) >> 1);
}

void CSetup::RealignText()
{
	int Visible;

	ItemsList->FontChanged();
	PtListBox->FontChanged();
	HdListBox->FontChanged();

	Visible = MiTimerLabel->IsVisible();
	MiTimerLabel->SetVisible(FALSE);
	MiKeyLabel->SetVisible(FALSE);
	MiHotkeyLabel->SetVisible(FALSE);
	MiPwdLabel->SetVisible(FALSE);

	MiTimerLabel->SetCaption(StrMiTimerLabel);
	MiKeyLabel->SetCaption(StrMiKeyLabel);
	MiHotkeyLabel->SetCaption(StrMiHotkeyLabel);
	MiPwdLabel->SetCaption(StrMiPwdLabel);

	MiTimerLabel->SetVisible(Visible);
	MiKeyLabel->SetVisible(Visible);
	MiHotkeyLabel->SetVisible(Visible);
	MiPwdLabel->SetVisible(Visible);
}

void CSetup::SetTimerSlider(int Timeout)
{
	int Value;

	if (Timeout >= 300)
		Value = (Timeout + 1530) / 30;
	else
		if (Timeout >= 60)
			Value = (Timeout + 615) / 15;
		else
			if (Timeout >= 30)
				Value = (Timeout + 30) / 2;
			else
				Value = Timeout;
	MiTimerSlider->SetValue(Value);
}

void CSetup::UnformatKeys(const unsigned char *Keys, char *UnformatStr)
{
	static const char *SpecialKeys = "abtnvfr";
	static const char *CursorKeys = "UDLR";
	int Index;

	for (Index = 0; *Keys; ++Keys, ++Index)
		if (*Keys >= 0x07 && *Keys <= 0x0d) {
			UnformatStr[Index++] = '\\';
			UnformatStr[Index] = SpecialKeys[*Keys - 7];
		}
		else
			if (*Keys >= 244) {
				UnformatStr[Index++] = '\\';
				UnformatStr[Index++] = 'F';
				if (*Keys <= 252)
					UnformatStr[Index] = *Keys - 195;
				else {
					UnformatStr[Index++] = '1';
					UnformatStr[Index] = *Keys - 205;
				}
			}
			else
				if (*Keys >= 191 && *Keys <= 194) {
					UnformatStr[Index++] = '\\';
					UnformatStr[Index] = CursorKeys[*Keys - 191];
				}
				else
					UnformatStr[Index] = *Keys;
	UnformatStr[Index] = '\0';
}

///////////////////
void CSetup::CloseBtnClick(CSetup *Setup)
{
	Setup->Hide();
	Setup->Loader->Focus();
}

void CSetup::HdListBoxSelect(CSetup *Setup, int ItemIndex)
{
	int Enabled;
	int HideStatus;

	HidingUpdate = FALSE;
	Setup->HdListBox->Enable();
	Setup->HdIgnoreAll->Enable();
	Setup->HdHideAll->Enable();
	Setup->HdRevealAll->Enable();
	Enabled = Setup->PartList->CanHide(ItemIndex);
	Setup->HdRadioGroup->SetEnabled(Enabled);
	HideStatus = Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex())->HidingItems[ItemIndex];
	Setup->HdRadioGroup->SetChecked(HideStatus);
	HidingUpdate = TRUE;
}

void CSetup::PtAddBtnClick(CSetup *Setup)
{
	TBootItem Entry;

	if (Setup->BootItems->GetCount() == 24) {
		Setup->Dialogs->ShowMessageDialog("Error: limit reached","XOSL cannot have more than 24 boot items");
		return;
	}

	memset(&Entry,0,sizeof (TBootItem));
	Setup->PtItemName->GetText(Entry.ItemName);
	Entry.PartIndex = Setup->PtListBox->GetItemIndex();
	Entry.Valid = 1;
	Entry.Activate = Setup->PartList->CanActivate(Entry.PartIndex);
	Entry.FixDriveNum = Setup->BootItems->CanFix(&Entry);
	Setup->BootItems->Add(Entry);
	Setup->ItemsList->Enable();
	Setup->ItemsList->AddRow();
	Setup->InitBootItems();
	Setup->ItemsList->SetItemIndex(Setup->ItemsList->GetCount() - 1);
	Setup->Loader->CreateBootList(0);
}

void CSetup::ItemsListSelect(CSetup *Setup, int ItemIndex)
{
	const TBootItem *Entry;
	char Unformat[64];

	if (ItemIndex == -1)
		Setup->InitNoItems();
	else {
		Setup->InitHideStatus();
		Setup->RemoveBtn->Enable();
		Setup->CloneBtn->Enable();
		Setup->MoveUpBtn->SetEnabled(ItemIndex);
		Setup->MoveDownBtn->SetEnabled(ItemIndex != Setup->ItemsList->GetCount() - 1);
		if (Setup->HdListBox->GetItemIndex())
			Setup->HdListBox->SetItemIndex(0);
		else {
			HdListBoxSelect(Setup,0);
			Setup->HdListBox->Refresh(); // ?
		}
		Entry = Setup->BootItems->GetBootItem(ItemIndex);
		Setup->PtItemName->SetText(Entry->ItemName);
		if (Entry->PartIndex != -1)
			Setup->PtListBox->SetItemIndex(Entry->PartIndex);
		else
			Setup->PtListBox->SetItemIndex(0);
		Setup->PtUpdateBtn->Enable();
		Setup->MiActivate->SetEnabled(Setup->PartList->CanActivate(Entry->PartIndex));
		Setup->MiActivate->SetChecked(Entry->Activate);
		Setup->MiInstBoot->Enable();
		UpdateIB = FALSE;
		Setup->MiInstBoot->SetChecked(Setup->BootItems->GetInstantBoot() == ItemIndex);
		UpdateIB = TRUE;
		Setup->MiPwdApply->Enable();
		Setup->MiKeyEdit->Enable();
		Setup->MiKeyApply->Enable();
		Setup->MiHotkeyEdit->Enable();
		Setup->MiHotkeyClearBtn->Enable();

		Setup->UnformatKeys(Entry->Keys,Unformat);
		Setup->MiKeyEdit->SetText(Unformat);
		if (!Entry->Hotkey)
			Setup->MiHotkeyEdit->SetText("");
		else {
			Setup->Keyboard->GetKeyName(Entry->Hotkey,Unformat);
			Setup->MiHotkeyEdit->SetText(Unformat);
		}
		if (Setup->BootItems->CanFix(Entry))
			Setup->MiDriveNumFix->Enable();
		else
			Setup->MiDriveNumFix->Disable();
		Setup->MiDriveNumFix->SetChecked(Entry->FixDriveNum);
	}
}

void CSetup::PtUpdateBtnClick(CSetup *Setup)
{
	TBootItem *Entry;
	int ItemIndex, PartIndex;

	ItemIndex = Setup->ItemsList->GetItemIndex();
	Entry = Setup->BootItems->GetBootItem(ItemIndex);
	Setup->PtItemName->GetText(Entry->ItemName);
	Entry->Valid = TRUE;
	PartIndex = Setup->PtListBox->GetItemIndex();
	if (Entry->PartIndex != PartIndex) {
		Entry->PartIndex = PartIndex;
		if (Setup->PartList->CanActivate(PartIndex))
			Entry->Activate = TRUE;
		else
			Entry->Activate = FALSE;
		if (Setup->BootItems->CanFix(Entry))
			Entry->FixDriveNum = 1;
		else
			Entry->FixDriveNum = 0;
		ItemsListSelect(Setup,ItemIndex);
	}
	Setup->InitBootItems();
	Setup->ItemsList->Refresh();
	Setup->Loader->CreateBootList(0);
}

void CSetup::MoveUpBtnClick(CSetup *Setup)
{
	int ItemIndex;

	ItemIndex = Setup->ItemsList->GetItemIndex();
	Setup->BootItems->MoveUp(ItemIndex);
	Setup->InitBootItems();
	Setup->ItemsList->SetItemIndex(ItemIndex - 1);
	Setup->Loader->CreateBootList(0);
}

void CSetup::MoveDownBtnClick(CSetup *Setup)
{
	int ItemIndex;

	ItemIndex = Setup->ItemsList->GetItemIndex();
	Setup->BootItems->MoveDown(ItemIndex);
	Setup->InitBootItems();
	Setup->ItemsList->SetItemIndex(ItemIndex + 1);
	Setup->Loader->CreateBootList(0);
}


void CSetup::MiActivateChange(CSetup *Setup, int Checked)
{
	Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex())->Activate = Checked;
	Setup->InitBootItems();
	Setup->ItemsList->Refresh();
}

void CSetup::MiDriveNumFixChange(CSetup *Setup, int Checked)
{
	Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex())->FixDriveNum = Checked;
}

void CSetup::MiInstBootChange(CSetup *Setup, int Checked)
{
	if (UpdateIB) {
		if (Checked)
			Setup->BootItems->SetInstantBoot(Setup->ItemsList->GetItemIndex());
		else
			Setup->BootItems->SetInstantBoot(-1);
		Setup->InitBootItems();
		Setup->ItemsList->Refresh();
	}
}

void CSetup::MiPwdApplyClick(CSetup *Setup)
{
	unsigned long Code;
	char Password[48];

	Setup->MiNewEdit->GetText(Password);
	Code = EncodePassword(Password);
	Setup->MiReEdit->GetText(Password);
	if (Code == EncodePassword(Password)) {
		Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex())->Password = Code;
		Setup->InitBootItems();
		Setup->ItemsList->Refresh();
	}
	else
		Setup->Dialogs->ShowMessageDialog("New password","Passwords do not match.");
}

void CSetup::MiTimerSliderChange(CSetup *Setup, int Value)
{
	CString Time;
	int Min, Sec;

	if (Value > 60)
		Value = 30 * Value - 1530;
	else
		if (Value > 44)
			Value = 15 * Value - 615;
		else
			if (Value > 30)
				Value = 2 * Value - 30;

	if (Value >= 60) {
		Min = Value / 60;
		Sec = Value % 60;
		Time = Min;
		if (Sec < 10)
			Time += ".0";
		else
			Time += ".";
		Time += Sec;
		Time += " min";
	}
	else {
		Time = Value;
		Time += " sec";
	}
	Setup->MiTimeout->SetCaption(Time);
	Setup->BootItems->SetTimeout(Value);
}

void CSetup::MiKeyApplyClick(CSetup *Setup)
{
	char UnformatStr[96];
	unsigned char *Keys;
	int Index, UFIndex;;
	int Key;

	Setup->MiKeyEdit->GetText(UnformatStr);
	Keys = Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex())->Keys;
	for (Index = 0, UFIndex = 0; Index < 15 && UnformatStr[UFIndex]; ++Index, ++UFIndex)
		if (UnformatStr[UFIndex] != '\\')
			Keys[Index] = UnformatStr[UFIndex];
		else
			switch (UnformatStr[++UFIndex]) {
				case 'a':
					Keys[Index] = 0x07;
					break;
				case 'b':
					Keys[Index] = 0x08;
					break;
				case 'f':
					Keys[Index] = 0x0c;
					break;
			case 'n':
					Keys[Index] = 0x0a;
					break;
				case 'r':
					Keys[Index] = 0x0d;
					break;
				case 't':
					Keys[Index] = 0x09;
					break;
				case 'v':
					Keys[Index] = 0x0b;
					break;
				case 'F':
					Key = UnformatStr[++UFIndex];
					if (Key >= '1' && Key <= '9')
						if (Key != '1')
							Keys[Index] = 195 + Key;
						else {
							Key = UnformatStr[++UFIndex];
							if (Key >= '0' && Key <= '2')
								Keys[Index] = 205 + Key;
							else {
								Keys[Index] = 244;
								--UFIndex;
							}
						}
					else {
						Keys[Index] = 'F';
						--UFIndex;
					}
					break;
				case 'U': // up
					Keys[Index] = 191;
					break;
				case 'D': // down
					Keys[Index] = 192;
					break;
				case 'L': // left
					Keys[Index] = 193;
					break;
				case 'R': // right
					Keys[Index] = 194;
					break;
				default:
					Keys[Index] = UnformatStr[UFIndex];
					break;
			}
	Keys[Index] = '\0';
	Setup->InitBootItems();
	Setup->ItemsList->Refresh();
}

void CSetup::MiHotkeyKeyPress(CSetup *Setup, int &Key)
{
	char KeyName[64];
	int ItemIndex;
	TBootItem *Entry;

	if (!Key)
		Setup->MiHotkeyEdit->SetText("");
	else {
		Setup->Keyboard->GetKeyName(KeyName);
		Setup->MiHotkeyEdit->SetText(KeyName);
	}
	ItemIndex = Setup->ItemsList->GetItemIndex();
	Entry = Setup->BootItems->GetBootItem(ItemIndex);
	Entry->Hotkey = Key;
	Key = KEY_END;
}

void CSetup::MiHotkeyClearBtnClick(CSetup *Setup)
{
	int Key = 0;

	Setup->MiHotkeyKeyPress(Setup,Key);
}

void CSetup::RemoveBtnClick(CSetup *Setup)
{
	int ItemIndex;

	ItemIndex = Setup->ItemsList->GetItemIndex();
	Setup->BootItems->Remove(ItemIndex);
	Setup->ItemsList->DeleteRow(ItemIndex);
	Setup->InitBootItems();
	if (Setup->ItemsList->GetCount())
		if (ItemIndex)
			Setup->ItemsList->SetItemIndex(0);
		else
			Setup->ItemsList->Refresh();
	else
		Setup->ItemsList->SetItemIndex(-1);

	Setup->Loader->CreateBootList(0);

}

void CSetup::CloneBtnClick(CSetup *Setup)
{
	TBootItem *Clone;

	if (Setup->BootItems->GetCount() == 24) {
		Setup->Dialogs->ShowMessageDialog("Error: limit reached","XOSL cannot have more than 24 boot items");
		return;
	}
	Clone = Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex());
	Setup->BootItems->Add(*Clone);
	Setup->ItemsList->Enable();
	Setup->ItemsList->AddRow();
	Setup->InitBootItems();
	Setup->ItemsList->SetItemIndex(Setup->ItemsList->GetCount() - 1);
	Setup->Loader->CreateBootList(0);
}

void CSetup::ResetBtnClick(CSetup *Setup)
{
	Setup->ResetData();
	Setup->Loader->CreateBootList(0);
}

void CSetup::SaveBtnClick(CSetup *Setup)
{
	Setup->BootItems->Save();
	Setup->Dialogs->ShowMessageDialog("Boot items","Data saved");
}

void CSetup::HdHidingSelect(CSetup *Setup, int ItemIndex)
{
	int HideStatus;
	int BIIndex, HIndex;

	if (!HidingUpdate)
		return;
	HideStatus = ItemIndex;
	BIIndex = Setup->ItemsList->GetItemIndex();
	HIndex = Setup->HdListBox->GetItemIndex();
	Setup->BootItems->GetBootItem(BIIndex)->HidingItems[HIndex] = HideStatus;
	Setup->InitHideStatus();
	Setup->HdListBox->Refresh();
}

void CSetup::HdIgnoreAllClick(CSetup *Setup)
{
	int Index, Count;
	char *HidingItems;
	TBootItem *Entry;

	HidingItems = Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex())->HidingItems;
	Count = Setup->PartList->GetCount();
	for (Index = 0; Index < Count; ++Index)
		HidingItems[Index] = ITEM_IGNORE;
	Setup->InitHideStatus();
	Setup->HdListBox->Refresh();
	HdListBoxSelect(Setup,Setup->HdListBox->GetItemIndex());
}

void CSetup::HdHideAllClick(CSetup *Setup)
{
	int Index, Count;
	char *HidingItems;
	TBootItem *Entry;

	HidingItems = Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex())->HidingItems;
	Count = Setup->PartList->GetCount();
	for (Index = 0; Index < Count; ++Index)
		if (Setup->PartList->CanHide(Index))
			HidingItems[Index] = ITEM_HIDE;
	Setup->InitHideStatus();
	Setup->HdListBox->Refresh();
	HdListBoxSelect(Setup,Setup->HdListBox->GetItemIndex());
}


void CSetup::HdRevealAllClick(CSetup *Setup)
{
	int Index, Count;
	char *HidingItems;
	TBootItem *Entry;

	HidingItems = Setup->BootItems->GetBootItem(Setup->ItemsList->GetItemIndex())->HidingItems;
	Count = Setup->PartList->GetCount();
	for (Index = 0; Index < Count; ++Index)
		if (Setup->PartList->CanHide(Index))
			HidingItems[Index] = ITEM_REVEAL;
	Setup->InitHideStatus();
	Setup->HdListBox->Refresh();
	HdListBoxSelect(Setup,Setup->HdListBox->GetItemIndex());
}


