/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#ifndef __setup__
#define __setup__

#include <gui.h>
#include <screen.h>
#include <graph.h>
#include <ptab.h>
#include <dialogs.h>
#include <items.h>
#include <loader.h>
#include <key.h>

#include <mem.h>
#include <encpwd.h>
#include <text.h>


class CSetup {
	public:
		CSetup(CLoader *LoaderToUse, CScreen *ScreenToUse,
			CPartList *PartListToUse, CBootItems *BootItemsToUse,
			CKeyboard *KeyboardToUse);
		~CSetup();

		void InitializeData();

		void Show();
		void Hide();
		int IsVisible();
		int GotFocus();

		void CenterWindow();
		void RealignText();
		void SetTimerSlider(int Timeout);
	private:
		CLoader *Loader;
		CScreen *Screen;
		CPartList *PartList;
		CBootItems *BootItems;
		CDialogs *Dialogs;
		CKeyboard *Keyboard;

	private:
		static int UpdateIB;
		static int HidingUpdate;


		static void CloseBtnClick(CSetup *Setup);
		static void MoveUpBtnClick(CSetup *Setup);
		static void MoveDownBtnClick(CSetup *Setup);
		static void RemoveBtnClick(CSetup *Setup);
		static void CloneBtnClick(CSetup *Setup);
		static void ResetBtnClick(CSetup *Setup);
		static void SaveBtnClick(CSetup *Setup);

		static void ItemsListSelect(CSetup *Setup, int ItemIndex);

		static void PtAddBtnClick(CSetup *Setup);
		static void PtUpdateBtnClick(CSetup *Setup);

		static void HdListBoxSelect(CSetup *Setup, int ItemIndex);
		static void HdHidingSelect(CSetup *Setup, int ItemIndex);
		static void HdIgnoreAllClick(CSetup *Setup);
		static void HdHideAllClick(CSetup *Setup);
		static void HdRevealAllClick(CSetup *Setup);

		static void MiActivateChange(CSetup *Setup, int Checked);
		static void MiDriveNumFixChange(CSetup *Setup, int Checked);
		static void MiInstBootChange(CSetup *Setup, int Checked);
		static void MiPwdApplyClick(CSetup *Setup);
		static void MiTimerSliderChange(CSetup *Setup, int Value);
		static void MiKeyApplyClick(CSetup *Setup);
		static void MiHotkeyKeyPress(CSetup *Setup, int &Key);
		static void MiHotkeyClearBtnClick(CSetup *Setup);


	private:

		// main controls
		CForm *Form;
		CListBox *ItemsList;
		CScrollBar *ItemsScroll;
		CButton *CloneBtn;
		CButton *RemoveBtn;
		CButton *MoveUpBtn;
		CButton *MoveDownBtn;
		CBevel *TabBevel;
		CTabControl *TabControl;
		CButton *ResetBtn;
		CButton *SaveBtn;
		CButton *CloseBtn;
		// partition
		CListBox *PtListBox;
		CScrollBar *PtScrollBar;
		CButton *PtAddBtn;
		CButton *PtUpdateBtn;
		CEdit *PtItemName;
		CLabel *PtLabel;
		// hiding
		CListBox *HdListBox;
		CScrollBar *HdScrollBar;
		CBevel *HdHideBevel;
		CRadioGroup *HdRadioGroup;
		CButton *HdIgnoreAll;
		CButton *HdHideAll;
		CButton *HdRevealAll;
		// Misc
		CBevel *MiTimerBevel;
		CLabel *MiTimerLabel;
		CTrackBar *MiTimerSlider;
		CLabel *MiTimeout;
		CBevel *MiKeyBevel;
		CLabel *MiKeyLabel;
		CEdit *MiKeyEdit;
		CButton *MiKeyApply;

		CBevel *MiHotkeyBevel;
		CLabel *MiHotkeyLabel;
		CEdit *MiHotkeyEdit;
		CButton *MiHotkeyClearBtn;

		CBevel *MiPwdBevel;
		CLabel *MiPwdLabel;
		CLabel *MiNewLabel;
		CLabel *MiReLabel;
		CEdit *MiNewEdit;
		CEdit *MiReEdit;
		CButton *MiPwdApply;
		CCheckBox *MiInstBoot;
		CCheckBox *MiActivate;
		CCheckBox *MiDriveNumFix;

		void ResetData();
		void InitBootItems();
		void InitHideStatus();
		void InitNoItems();

		void UnformatKeys(const unsigned char *Keys, char *UnformatStr);

	private:
		void CreateControls();
		void InitializeControls();
		void InstallControls();
		void InitPartitions();
		void InitHiding();

		//
		void AddDisk(CListBox *ListBox, int Row, int Drive);
		void AddType(CListBox *ListBox, int Row, int Type);
		void AddSystem(CListBox *ListBox, int Row, const char *FSName);
		void AddSize(CListBox *ListBox, int Row, long Sectors);
		//
		//
		int BootItemsDrawn;
		int PartitionDrawn;
};



#endif