/*
 * Extended Operating System Loader (XOSL)
 * Copyright (c) 1999 by Geurt Vos
 *
 * This code is distributed under GNU General Public License (GPL)
 *
 * The full text of the license can be found in the GPL.TXT file,
 * or at http://www.gnu.org
 */

#ifndef __XOSL_Application__
#define __XOSL_Application__

#include <newdefs.h>
#include <capture.h>

#include <mouse.h>
#include <graph.h>
#include <key.h>
#include <screen.h>
#include <fs.h>

#include <loader.h>
#include <about.h>
#include <setup.h>
#include <setup.h>
#include <prefer.h>
#include <dialogs.h>

class CApplication {
	public:
		CApplication();
		~CApplication();
		int Execute();
	private:
		CMouse *Mouse;
		CKeyboard *Keyboard;
		CFileSystem *FileSystem;
		CPartList *PartList;

		CScreen *Screen;

		CXOSLData *XOSLData;
		CBootItems *BootItems;

		CLoader * Loader;
		CAbout *About;
		CSetup *Setup;
		CPreference *Preference;

		CDialogs *Dialogs;

		void CriticalError(const char *ErrorMsg);
		void ExecuteBypass();
		//
		void CreateCoreClasses();
		void DestroyCoreClasses();
		void InitializeCore();
		void FinalizeCore();
		//
		void CreateApplication();
		void DestroyApplication();
		//
		void LoadDefaultFont();
		void LoadXOSLSplashLogo();
		void LoadXOSLLogo();
		void LoadSplashLogo();
		void LoadAdditionalFont();
		void LoadWallpaper();

		void InitializeMouse();
		void InitAppGraphics();

		void DisplayBootError();

		void Shutdown();
		void Restart();
		void Reboot();

		static void ShutDownReboot(CApplication *Application);

      void CycleWindow();

		void DoSplashFadeOut();
		void DoAppFadeIn();
		int HandleInstantBoot();

#ifdef DOS_DEBUG
		void ScanForDebugKeys(int Key);
#endif

		CTextCapture *TextCapture;

};

#endif